######################################################################
#
# IMAPFolder, manage IMAP Folders for Webmail product
#
# This software is governed by a license. See 
# LICENSE.txt for the terms of this license.
#
######################################################################

from IMAPMessage import *
import IMAPPasserelle
import POPPasserelle
from MessageTools import parseMessage, parseHeader
from SimpleCache import SimpleCache

class IMAPFolder:
    """ Implemetation of the IMAP MailBox """

    def __init__(self, IMAPName, readableName, webMail, type='inbox'):
        """ constructor """

        self.IMAPName = IMAPName
        self.readableName = readableName
        self.type = type
        self.webMail = webMail


    def getImapName(self):
        """ return the imap name """
        
        return self.IMAPName

    def getReadableName(self):
        """ return the readable name """
        return self.readableName

    def getWebMail(self):
        """ return WebMail instance """

        return self.webMail
        
    def moveIMAPMessages(self, folderdest, imapids, copy=1, con = None):
        """ move the message imapid to the dolderdest folder """
        if folderdest == "deleting":
            # move message to trash
            _folderNameDest = self.getWebMail().getTrashFolder().getImapName()
        else:
            # move message to folder folderdest
            _folderNameDest = folderdest
        if con is None:
            con=self.getWebMail().getPasserelle()
            con.connect(self.getWebMail(), server=self.getWebMail().getIMAPServer(), port=self.getWebMail().getIMAPPort())
            con.login(self.getWebMail().getIMAPLogin(), self.getWebMail().getIMAPPassword())

        if not copy:
            # delete trash message, no copy to another folder
            con.selectFolder(self.getImapName())
            
        res = 0
        for IMAPId in imapids:
            if copy:
                # move message to folder _folderNameDest
                res = con.copy(folderNameSrc = self.getImapName(),
                               folderNameDest = _folderNameDest,
                               IMAPId = IMAPId)
            else:
                res = 1

            if res == 1 or (res == 2 and folderdest == "deleting"):
                # if copy is sucessfull or we want delete message when quota is over, we delete message
                con.setFlag(IMAPId = IMAPId, flag = "delete")
        con.expunge()
        

    def getNumberOfIMAPMessage(self, REQUEST=None, con = None):
        """ return the number of mail in this mailbox """
        if con is None:
            con=self.getWebMail().getPasserelle()
            con.connect(self.getWebMail(), server=self.getWebMail().getIMAPServer(), port=self.getWebMail().getIMAPPort())
            con.login(self.getWebMail().getIMAPLogin(), self.getWebMail().getIMAPPassword())
        
        sortmail = self.getWebMail().getSortMail(REQUEST)
        if string.find(sortmail, "search") != -1:
            #number of messages from a search in one folder
            res = con.getNumberOfMessage(self.getImapName(), search = sortmail)
        else:
           #number of messages in the folder
            res = con.getNumberOfMessage(self.getImapName())
        return res
    
    def getIMAPMessagesHeaders(self, sortmail = "", start = 1, end = 0, listing_size = 0, REQUEST = None, RESPONSE = None, con = None):
        """
        return the liste of headers
        between start and end
        or
        between start and start + listingsize
        with the sort sort sortmail 
        """
        parsed_res = []

        self.getWebMail().setSortMail(sortmail, REQUEST,RESPONSE)
        if con is None:
            con = self.getWebMail().getPasserelle()
            con.connect(self.getWebMail(), server=self.getWebMail().getIMAPServer(), port=self.getWebMail().getIMAPPort())
            con.login(self.getWebMail().getIMAPLogin(), self.getWebMail().getIMAPPassword())
        res = con.listMessagesHeaders(folderName=self.getImapName(), sortmail=self.getWebMail().getSortMail(REQUEST), start=start, end=end, listing_size=listing_size)
        
        for header in res:
            parsed_res.append(parseHeader(header, self.getWebMail().getNbCharSubject()))

        return parsed_res


    def getIMAPMessage(self, imapid="", con = None):
        """ """
        if con is None:
            con = self.getWebMail().getPasserelle()
            con.connect(self.getWebMail(), server=self.getWebMail().getIMAPServer(), port=self.getWebMail().getIMAPPort())
            con.login(self.getWebMail().getIMAPLogin(), self.getWebMail().getIMAPPassword())
        
        res, direct_body, flags = con.getMessage(folderName = self.getImapName(), IMAPId = imapid)
        
        imap_message = parseMessage(mess = res, direct_body = direct_body, flags = flags, imapid = imapid)
        return imap_message


    def getRawIMAPMessage(self,imapid, con = None):
        """ return the message without parsing"""
        if con is None:
            con=self.getWebMail().getPasserelle()
            con.connect(self.getWebMail(), server=self.getWebMail().getIMAPServer(), port=self.getWebMail().getIMAPPort())
            con.login(self.getWebMail().getIMAPLogin(), self.getWebMail().getIMAPPassword())
        res, direct_body, flags = con.getMessage(folderName = self.getImapName(), IMAPId = imapid)
        return res


    def getType(self):
        """ return the type of this folder """

    def emptyTrash(self):
        """
        delete all the message of this folder
        if this folder'stype is trash
        """
