######################################################################
#
# IMAPPasserelle, api to IMAPLibLocal
#
# This software is governed by a license. See 
# LICENSE.txt for the terms of this license.
#
######################################################################

import timeoutsocket
import thread, sys
from IMAPMessage import *

try:
    import email
except:
    pass

# Check python version
if sys.version[:3] == '2.1':
    import imapLibLocal
else:
    import imaplib
    imapLibLocal = imaplib

import string, re, time
from Globals import Persistent
from SimpleCache import SimpleCache, keepAlive_IMAP, default_cache_timeout

timeoutsocket.setDefaultSocketTimeout(20) # 20 seconds de timeout sur les sockets


class IMAPPasserelle:
    """ IMAP gateway """

    def __init__(self, conAlive, cache_buffer = None):
        """ Constructor """
        if cache_buffer is not None:
            self._cache = cache_buffer
        else:
            self._cache = SimpleCache(timeout = default_cache_timeout)
        self.keepAlive_IMAP = conAlive
        
    def connect(self, webmail = None, server = "", port = ""):
        """ connet to IMAP server"""
        # If disallowKeepAlive is set by login method send the message
        # to the webmail object
        if hasattr(self, '_disallowKeepAlive') and webmail is not None:
            webmail.setConnexionAlive(0)
        try:
            try:
                # Test connexion validity
                if self.connexionAlive():
                    self.connection.noop()
                    need_reconnection = 0
                else:
                    need_reconnection = 1
            except:
                need_reconnection = 1
            if need_reconnection:
                if port is not None and port != "":
                    self.connection = imapLibLocal.IMAP4(server, int(port))
                else:
                    self.connection = imapLibLocal.IMAP4(server)
        except:
            try:
                time.sleep(3)
                if port is not None and port != "":
                    self.connection = imapLibLocal.IMAP4(server, int(port))
                else:
                    self.connection = imapLibLocal.IMAP4(server)
            except:
                if webmail is not None:
                    raise 'Redirect', webmail.absolute_url() + '/view_error_dtml'
                return 0
        return 1
    
    def connexionAlive(self):
        """ Return the connexion type Alive or not """
        if hasattr(self, 'keepAlive_IMAP'):
            return self.keepAlive_IMAP
        return keepAlive_IMAP

    def _setStamp(self, stamp):
        """ set the stamp of this gateway """
        self._currentStamp = stamp

    def _getStamp(self):
        """ return the stamp of the gateaway """
        return self._currentStamp

    def login(self, login, password):
        """ login to IMAP server """
        res = ['NONOK']
        if self.connection.state == 'NONAUTH':
            try:
                res = self.connection.login(login, password)
            except imapLibLocal.IMAP4.abort, msg:
                # disallow permanente connexion
                # 'cause some IMAP server maybe don't
                # support it.
                self._disallowKeepAlive = 1
                self.keepAlive_IMAP = 0
        else:
            self.connection.select()
            return
        
        if res[0] != "OK":
            return "LOG_FAILED"
        try:
            self.connection.select()
        except:
            return "MAILBOX_READ_ONLY"


    def logout(self):
        """ logout to IMAP server """
        if not self.connexionAlive():
            self.connection.logout()

    def listFolders(self):
        """ return the liste of IMAP folders """
        if self._cache.get('listFolders'):
            return self._cache.get('listFolders')

        list = self.connection.list()[1]
        IMAPFolders = []
        for item in list:
            if item is not None:
                try:
                    vl = re.sub(r'(\(.+\))(.+)', r'\2', item)
                    val = string.split(vl, ' ', 2)[2]
                    val = string.strip(string.replace(val, '"', ''))
                    IMAPFolders.append(val)
                except:
                    IMAPFolders.append("folder error")
                    
        self._cache.set('listFolders', IMAPFolders)
        return IMAPFolders


    def selectFolder(self, name):
        """ select a IMAP folder """
        nb_mails = self.connection.select(name)[1][0]

        if self._cache.get('getIMAPList') and not self._cache.get('currentFolder', name) or not self._cache.get('currentNbMails', nb_mails):
            self._cache.remove('getIMAPList')
            self._cache.set('currentFolder', name)
            self._cache.set('currentNbMails', nb_mails)
        

    def createFolder(self, name):
        """ create a new IMAP folder """

        #encodage utf-7
        encoded_name = string.replace(name, "&", "&-")
        encoded_name = string.replace(encoded_name, "", "&AOk-")
        encoded_name = string.replace(encoded_name, "", "&AOI-")
        encoded_name = string.replace(encoded_name, "", "&AOA-")
        encoded_name = string.replace(encoded_name, "", "&AOg-")
        encoded_name = string.replace(encoded_name, "", "&AOc-")
        encoded_name = string.replace(encoded_name, "", "&APk-")
        encoded_name = string.replace(encoded_name,  "", "&AOo-")
        encoded_name = string.replace(encoded_name,  "", "&AO4-")
        encoded_name = string.replace(encoded_name, "", "&APM-")
        encoded_name = string.replace(encoded_name,  "", "&APE-")
        encoded_name = string.replace(encoded_name,  "", "&AOE-")
        encoded_name = string.replace(encoded_name,  "", "&APQ-")
        encoded_name = string.replace(encoded_name,  "", "&AMk-")
        name = string.replace(encoded_name, "", "&AOs-")
        
        fold_name = name

        name = '"INBOX.' + name + '"'
        try:
            if fold_name != "inbox" and fold_name != "INBOX":
                self._cache.remove('listFolders')
                return self.connection.create(name)
            else:
                return "NO"
        except:
            return "NO"


    def deleteFolder(self, wmail, name):
        """ delete IMAP folder """
        if name not in["INBOX", '"INBOX"', 'inbox', '"inbox"', wmail.getDraftIMAPName(), wmail.getTrashIMAPName(), wmail.getSentMailIMAPName()]:
            delFolder = self.connection.delete('"'+name+'"')

        try:
            if delFolder[0]=="OK":
                self._cache.remove('listFolders')
                return 1
            else:
                return 0
        except:
            return 0


    def getIMAPList(self, folderName, sortmail, start=0, how_many="ALL"):
        """ return the sorted list of IMAP Id """
        self.selectFolder(folderName)
        if self._cache.get('getIMAPList') and self._cache.get('currentSortMail', sortmail):
            return self._cache.get('getIMAPList')

        self._cache.set('currentSortMail', sortmail)
        
        inverse_result=0
        if sortmail=="date":
            #get messages list on server, no sort
            result = self.connection.uid('SEARCH', how_many)
            inverse_result=1
        elif sortmail in ["DATE", "SUBJECT", "FROM", "SIZE", "TO"]:
            #get sorted messages list
            if sortmail in ["DATE", "SIZE"]:
                inverse_result=1
            if sortmail == "DATE":
                sortmail="(DATE)"
            else:
                sortmail="("+sortmail+" DATE)"
    
            result=self.connection.uid('SORT', sortmail,'iso-8859-1', how_many)
            
        elif string.find(sortmail, "search") != (-1):
            #get messages list from search
            sortmail_list=string.split(sortmail, "x2jq")
            search_where=sortmail_list[1]
            search_line=string.replace(sortmail_list[2], "zz20", " ")
            search_when=sortmail_list[3]
            search_date=sortmail_list[4]
            flagged=0
            if sortmail_list[5]=="oui":
                flagged=1
            sort=sortmail_list[-1]

            if sort not in ["SUBJECT", "FROM", "TO"]: 
                inverse_result=1

            if flagged:
                result=self.connection.uid('SORT', '('+sort+')','iso-8859-1', search_where, search_line, search_when, search_date, "FLAGGED")
            else:
                result=self.connection.uid('SORT', '('+sort+')','iso-8859-1', search_where, search_line, search_when, search_date)

            nb=string.split(result[-1][0])
            nb_messages_in_mailbox=len(nb)

        elif sortmail=="FLAGGED":
            result = self.connection.uid('SEARCH', 'FLAGGED')
            inverse_result=1
        else:
            result = self.connection.uid('SEARCH', how_many)
            inverse_result=1

        # set imap list
        liste_res=map(int, string.split(result[-1][-1]))

        if inverse_result:
            liste_res.reverse()  
        
        self._cache.set('getIMAPList', liste_res)

        return liste_res
        

    def listMessagesHeaders(self, folderName = "INBOX", sortmail = "", start = 0, end = 0, listing_size = 0):
        """ return headers from a IMAP folder """
        start = int(start)
        if not end:
            end = start + int(listing_size)
        end = int(end)

        liste_res = self.getIMAPList(folderName = folderName, sortmail = sortmail, start = start)

        #limit the number of message in the list to number of messages by page (choose in options)
        liste_res = liste_res[start:end]

        liste_messages = [] 
        for IMAPId in liste_res:
            #get flags, size and header for message
            rep = self.connection.uid('FETCH',IMAPId,'(FLAGS RFC822.SIZE RFC822.HEADER)')
            val= rep[-1][0][0]
            flags = re.sub(r'(.+)(FLAGS \()(.+)(\).+)', r'\3', val)
            if string.find(flags, 'UID') != (-1):
                flags = " "
            taille = int(re.sub(r'(.+)(RFC822.SIZE )(\d+)( .+)', r'\3 ', val))
            liste_messages.append({'message':rep[-1][0][1], 'size':taille, 'flags':flags, 'imapId':IMAPId})
        return liste_messages


    def getFlags(self, folderName, IMAPId):
        """ return flags of a message """
        
        self.selectFolder(folderName)
        rep = self.connection.uid('FETCH', str(IMAPId), '(FLAGS)')
        val = rep[-1][0]
        flags = re.sub(r'(.+)(FLAGS \()(.+)(\).+)', r'\3', val)
        if string.find(flags, 'UID') != (-1):
            flags = " "
        return flags

    def setFlag(self, folderName = "", IMAPId = "", flag = ""):
        """ set flag of a message """

        if folderName:
            self.selectFolder(folderName)
            
        if flag == "delete":
            self.connection.uid('STORE', str(IMAPId), '+FLAGS', '(\Deleted)')
        if flag == "anwser":
            self.connection.uid('STORE', str(IMAPId), '+FLAGS', '(\Answered)')
        if flag == "+flagged":
            self.connection.uid('STORE', str(IMAPId), '+FLAGS', '(\Flagged)')
        if flag == "-flagged":
            self.connection.uid('STORE', str(IMAPId), '-FLAGS', '(\Flagged)')
        if flag == "draft":
            self.connection.uid('STORE', str(IMAPId), '+FLAGS', '(\Draft)')
            

    def getMessage(self, folderName = "INBOX", IMAPId = ""):
        """ get full message """
        
        if folderName:
            self.connection.select(folderName)

        direct_body = ' '
        flags = ""
        
        try:
            direct_body = self.connection.uid('FETCH', str(IMAPId), '(BODY.PEEK[1])')[-1][0][-1]
        except:
            pass
        try:
            message = self.connection.uid('FETCH', str(IMAPId), 'RFC822')[-1][0][1]
       	except:
            return 'DELETED', direct_body, flags
        try:
            flags = self.getFlags(folderName, IMAPId)
        except:
            pass
       	return message, direct_body, flags


    def getPreviousAndNextMessagesIds(self, folderName, sortmail, IMAPId):
        """ get next message on folder (sorted)  """
        
        res = self.getIMAPList(folderName, sortmail)
        try:
            index = res.index(int(IMAPId))
        except:
            return None, None
        
        index_next = index + 1
        index_prev = index - 1
        prev = None
        if index_prev >= 0:
            prev = res[index_prev]
        try:
            next = res[index_next]
        except:
            next = None

        return prev, next
        

    def copy(self, folderNameSrc, folderNameDest, IMAPId):
        """ copy a message from a folder to another """
        
        copy_ok = 0

        if folderNameSrc:
            self.selectFolder(folderNameSrc)
            _res = self.connection.uid('COPY', str(IMAPId), '"' + folderNameDest + '"')
            try:
                if _res[1][0] == "Over quota":
                    copy_ok = 2
            except:
                pass
            res = _res[0]
            if res == "OK" :
                copy_ok = 1
                self._cache.remove('getIMAPList')
        
        return copy_ok

    def getNumberOfRecentMails(self, foldername = 'INBOX'):
        """ return the number of new mail """
        try:
            typ, data = self.connection.select(foldername, 1)
            typ, data = self.connection.search(None, 'UNSEEN')
            n =  len(data[0].split())
            typ, data = self.connection.search(None, 'NEW')
            m = len(data[0].split())
            return m+n
        except:
            return 0
    
    def getQuota(self):
        """ return % used quota on mailbox """
        typ, dat = self.connection._simple_command("GETQUOTAROOT", "INBOX")
        res = self.connection.untagged_responses['QUOTA']
        # Removing localImapLib but getquotaroot imaplib don't run 
        #res = self.connection.get_quota_root()
        tab = string.split(res[0])
        return (100*int(tab[-2]))/int(string.replace(tab[-1], ")", ""))

    def getNumberOfMessage(self, folderName, search = ""):
        """ return the number of message in a folder """
        
        if not search:
            #number of messages in folder
            return self.connection.select(folderName)[1][0]
        else:
            #number of messages from a search
            if folderName:
                self.connection.select(folderName)
            sortmail_list = string.split(search, "x2jq")
            search_where = sortmail_list[1]
            search_line = string.replace(sortmail_list[2], "zz20", " ")
            search_when = sortmail_list[3]
            search_date = sortmail_list[4]
            flagged = 0
            if sortmail_list[5] == "oui":
                flagged = 1
                
            sort = sortmail_list[-1]
            if flagged:
                result = self.connection.uid('SORT', '(' + sort + ')', 'iso-8859-1', search_where, search_line, search_when, search_date, "FLAGGED")
            else:
                result = self.connection.uid('SORT', '(' + sort + ')', 'iso-8859-1', search_where, search_line, search_when, search_date)

            return len(string.split(result[-1][0]))
            

    def expunge(self):
        """ expunge ! """

        #delete on IMAP server all messages where flag deleted is on
        self.connection.expunge()

    def writeMessage(self, folderName, raw_message):
        """ wrtite a message in a folder """

        res = self.connection.append(folderName, None , None, raw_message)
        try:
            return string.replace(string.split(res[1][0])[2], ']', '')
        except:
            return ""
        
    def getBodyStructure(self, IMAPId):
        """ get the body structure of the message """

        a=1
        #body=self.connection.uid('FETCH',str(IMAPId), '(BODY.PEEK[1])')[-1][0][-1]
##        val=self.connection.uid('FETCH',str(IMAPId), 'BODY')
##        print " 1******", val
##        print "3****", val[-1][0]
        
##        val=str(val[-1][0])

##        for item in string.split(self.connection.uid('FETCH',str(IMAPId), 'BODYSTRUCTURE') [-1][0], ')('):
##            print item
##        print ' '
##        print "222222222222222222222222222222"
        print ' '
        nbParts=0
        v0=self.connection.uid('FETCH',str(IMAPId), 'BODY')
        v=str(v0[-1][0])

        liste=[]
        print v
        v=v[string.find(v, '("')+1:]
        for item in string.split(v, ')('):
            #print ' '
            #print item
            item2=string.replace(item, "(", "")
            item2=string.replace(item2, ")", "")

            v1=string.split(item2, '"')
            #print "v1"
            #print v1
            #print " "

        
            v2=string.split(item2)
            #print "v2", v2
            liste.append(item2)
            nbParts=nbParts+1
            print " "
            print "type = "+self._strip2Quotes(v2[0])+"/"+self._strip2Quotes(v2[1])

            #"7bit" , "quoted-printable", "base64", "8bit","binary"
           
            if string.find(item, '"FILENAME"')!=(-1):
                ind=v1.index('FILENAME')
                print "filename =", self._strip2Quotes(v1[ind+2])
            elif string.find(item, '"NAME"')!=(-1):
                ind=v1.index('NAME')
                print "name =", self._strip2Quotes(v1[ind+2])
                
            if string.find(item, '"BASE64"')!=(-1):
                ind=v2.index('"BASE64"')
                print "taille (base64) =", self._strip2Quotes(v2[ind+1])
            elif string.find(item, '"7BIT"')!=(-1):
                ind=v2.index('"7BIT"')
                print "taille (7bit) =", self._strip2Quotes(v2[ind+1])
            elif string.find(item, '"8BIT"')!=(-1):
                ind=v2.index('"8BIT"')
                print "taille (8bit) =", self._strip2Quotes(v2[ind+1])
            elif string.find(item, '"QUOTED-PRINTABLE"')!=(-1):
                ind=v2.index('"QUOTED-PRINTABLE"')
                print "taille (quoted) =", self._strip2Quotes(v2[ind+1])
            elif string.find(item, '"BINARY"')!=(-1):
                ind=v2.index('"BINARY"')
                print "taille (binary) =", self._strip2Quotes(v2[ind+1])
            

        print "il y a ", nbParts, "parts"
        #print liste


    def _strip2Quotes(self, s):
        """ Strip double quotes around string <s> if any."""
        
        try:
            if s[0]==s[-1]=='"':
                s = s[1:-1]
        except:
            pass
        return s
