######################################################################
#
# MessagesTools, messages and headers parser for WebMail product
#
# This software is governed by a license. See 
# LICENSE.txt for the terms of this license.
#
######################################################################


import string
import re
import IMAPMessage
import Attachement

DEFAULT_CHARSET = 'iso-8859-1'

try:
	import email, email.Errors, email.Header, email.Message, email.Utils, time, email.Parser, email.Charset
	EMAIL_LIB = 1
except:
	EMAIL_LIB = 0
try:
	import rfc822, cStringIO, StringIO, base64, mimify, mimetools, multifile, mime_message
	import DateTime
except:
	pass

def parseHeader(header, nbCharSubject = 25):
	if EMAIL_LIB:
		try:
			return parseHeaderMail(header, nbCharSubject)
			
		except:
			pass
	
	return parse_RFCHeaders(header, nbCharSubject)

def parseMessage(mess, direct_body, flags, imapid):
	if EMAIL_LIB:
		try:
			return parseMessageMail(mess, direct_body, flags, imapid)
		except:
			pass
	
	return parse_RFCMessage(mess, direct_body, flags, imapid)


def parseHeaderMail(header, nbCharSubject = 25):
	""" return dictionnary of parsed header """

	message = header['message']
	size    = header['size']
	flags   = header['flags']
	imapId  = header['imapId']

	# Build a Message
	msg = email.message_from_string(message)
	
	# Flag Attachement presence
	attachment = 0
	if msg.get_boundary():
		attachment = 1
		
	# Date Format
	date = msg.get('Date', 'Thu, 18 Sep 2002 12:02:27 +1000')
	date = render_date(date)
	
	# Subject Format without Charset
	subject = msg.get('Subject', '[pas de sujet]')
	subject = render_subject(subject)
	
	# Sender Format
	From = msg.get('From', 'Expditeur non spcifi')
	From = email.Utils.parseaddr(From)
	From = ( email.Header.decode_header(From[0])[0][0], From[1])
	
	# Reply
	ReplyTo = msg.get('Reply-To', '')
	ReplyTo = email.Utils.parseaddr(ReplyTo)

	# Recipents
	To = msg.get('To', '')
	To = parse_Addresses(To)
	if To:
		To = To.split(',')[0]

	Exp = [None, None]
	# Reel name, email
	Exp[0], Exp[1] = From[0], From[1]

	# Sender Display
	sender = Exp[0] or Exp[1]
	    
	# Render readable size
	size = render_size(size) 
	    
	head = {'imap_id': imapId,
		'sender': string.capitalize(sender[0:nbCharSubject]),
		'mail_sender': Exp[1],
		'to': To[0:nbCharSubject],
		'replyTo': ReplyTo,
		'size': size,
		'sujet': string.capitalize(subject),
		'date': date,
		'read': 0,
		'answered': 0,
		'deleted': 0,
		'flagged': 0,
		'attachments':attachment
		}

	# Build Flags settings
	flags = flags.lower()
	if string.find(flags, 'seen') != -1:
		head['read'] = 1
	if string.find(flags, 'deleted') != -1:
		head['deleted'] = 1
	if string.find(flags, 'answered') != -1:
		head['answered'] = 1
	if string.find(flags, 'flagged') != -1:
		head['flagged'] = 1

	return head

def parseMessageMail(mess, direct_body, flags, imapid):
	""" return message instance of IMAPMessage """

	# Build a Message
	try:
		msg = email.message_from_string(mess)
	except:
		msg = email.Parser.Parser().parsestr(mess, headersonly = 1)

	date = msg.get('Date', 'Thu, 18 Sep 2002 12:02:27 +1000')
	date = render_date(date)

	subject = msg.get('Subject', '[pas de sujet]')
	subject = render_subject(subject)

	From = msg.get('From', 'Expditeur non spcifi')
	From = email.Utils.parseaddr(From)
	From = ( email.Header.decode_header(From[0])[0][0] or 'Expditeur non spcifi', From[1])
	msgnum = int(imapid)
	nb, raw_attachments = walk_parts(msg, From, date, 0, msgnum)

	attachements = []
	for item in raw_attachments:
	    attachements.append(Attachement.Attachement(id = item['id'],
							filename = item['filename'],
							content_type = item['content_type'],
							size = item['size'],
							data = item['data']
							)
				)
	body = ''
	content_type = 'text/plain'
	charset = DEFAULT_CHARSET
	if msg.is_multipart():
	    for item in msg.walk():
		    if string.find(item.get_content_type(), 'text/plain') != -1 and not body:
			    body = item
	    if body:
		    content_type = body.get_content_type()
		    charset = body.get_content_charset()
		    body = body.get_payload(decode = 1)
		    
	else:
		content_type = msg.get_content_type()
		charset = msg.get_content_charset()
		body = msg.get_payload(decode = 1)

	if not body:
		body = email.message_from_string(direct_body)
		charset = body.get_content_charset()
		content_type = body.get_content_type()
		body = body.get_payload(decode = 1)
	if charset:
		charset = charset.lower()
	if charset and charset != DEFAULT_CHARSET:
		try:
			body = body.decode(charset)
			body = body.encode(DEFAULT_CHARSET)
		except:
			pass

	headers = {}
	for k, v in msg.items():
	    headers[k] = v
	headers['Content-Type'] = content_type

	_to = parse_Addresses(headers.get('To', ''))
	_cc = parse_Addresses(headers.get('Cc', ''))

	destinataires = {'to': _to, 'cc': _cc, 'bcc': []}

	dico_flags = {'read': 0, 'answered': 0, 'deleted': 0, 'flagged': 0}
	flags = flags.lower()
	for item in ['read', 'answered', 'deleted', 'flagged']:
	    if string.find(flags, item) != -1:
		dico_flags[item] = 1

	ref = IMAPMessage.IMAPMessage(IMAPId = imapid,
				      flags = dico_flags,
				      expediteur = From,
				      size = 0,
				      date = date,
				      subject = subject,
				      body = body,
				      destinataires = destinataires,
				      replyTo = '',
				      attachements = attachements,
				      headers = headers
				      )
        return ref


def encoded_part(part, charset = DEFAULT_CHARSET):
	""" Return an encoded charset subject """
	s = email.Header.make_header([(part, charset)])
	return str(s)

def encoded_addr(taddr):
	""" return an well encoding addresse of the tuple taddr : (Real Name, email)"""
	return email.Utils.formataddr(taddr)

def parse_Addresses(addresses):
    """ return a string which is well parsed """
    adds = addresses
    adds = [adds]
    adds = email.Utils.getaddresses(adds)
    adds = map(lambda x: (email.Header.decode_header(x[0])[0][0], x[1]), adds)
    adds = map(lambda x: email.Utils.formataddr(x), adds)
    adds = string.join(adds, ', ')
    return adds

def render_subject(subject):
	"""Return a rendered subject """
	if EMAIL_LIB:
		try:
			return string.join(map(lambda x:x[0], email.Header.decode_header(subject)))
		except:
			return subject
	else:
		return RFC_render_subject(subject)

def render_size(s):
	""" Return a formated size in bytes or MBytes """
	if s > 1024*1024:
		size = "%.2f Mb" % (s / 1024.0 / 1024)
	else:
		size = "%.2f Kb" % (s / 1024.0)
	return size

def render_date(date):
	""" Render a formated Date """
	try:
		if EMAIL_LIB:
			return time.strftime('%d/%m/%Y %H:%M', email.Utils.parsedate(date))
		else:
			return RFC_render_date(date)
	except:
		return date

def gen_filename(name, mtyp, addr, date, n):
	if not name:
		suf = name or mtyp == 'text/plain' and 'txt' or 'xxx'
		file = 'attachment %d.%s' % (n, suf)
	else:
		file = email.Header.decode_header(name)[0][0]
		if type(file) is not type(''):
			file = name
        return file

def walk_parts(msg, addr, date, count, msgnum):
        l  = []
	params = []
	for part in msg.walk():
		
		if part.is_multipart():	
			if part.get_content_maintype() != 'multipart':
				params = part.get_params()
			continue
		# --129.78.111.142.126.24561.1032609041.111.5183
		# Content-Type: image/jpeg; name=K-Woolyman-swimming.jpg
		# Content-Transfer-Encoding: base64
		# [Content-Disposition: attachment; filename="competition.ps"]
		
		dtypes = part.get_params(None, 'Content-Disposition') or params
		if not dtypes:
			
			if part.get_content_type() == 'text/plain':
				continue
			
			ctypes = part.get_params()
			
			if not ctypes:
				continue
			for key,val in ctypes:
				if key.lower() == 'name':
					filename = gen_filename(val, part.get_content_type(), addr, date, count)
					break
			else:
				filename = 'attachment'
			params = []
		else:
			
			attachment, filename = None, None
			for key, val in dtypes:				
				key = key.lower()
				if key in ['filename', 'name', 'boundary']:
					if type(val) is type(''):
						filename = val
					elif val:
						filename = val[-1]
					attachment = 1
				if key == 'attachment':
					attachment = 1
			
			#if not attachment:				
			#	continue
			params = []
			filename = gen_filename(filename, part.get_content_type(), addr, date, count)
			
		try:
			data = part.get_payload(decode=1)
		except:
			continue

		if not data:
			continue
		else:
			params = []

		#if type(data) is type(msg):
		#	count, t = walk_parts(data, addr, date, count, msgnum)
                #        l = l + t
		#	continue

                _size = render_size(len(data))
          
                l.append({'id': count, 'filename': filename, 'content_type': part.get_content_type(), 'size': _size, 'data': data})
		count += 1
	
	return count, l


######################################################################################
#
# OLD Verision of Parsed Message
#
# Based on mimify lib (soon deprecated)
#
######################################################################################

is_quoted_printable = re.compile('=\\?iso-8859-[12]\\?q\\?([^? \t\n]+)\\?=', re.I)
is_base64 = re.compile('=\\?iso-8859-[12]\\?b\\?([^? \t\n]+)\\?=', re.I)

def parse_RFCHeaders(header, nbCharSubject=25):
    """ return dictionnary of parsed header """

    message=header['message']
    size=header['size']
    flags=header['flags']
    imapId=header['imapId']

    fp=StringIO.StringIO(message)
    
    val = rfc822.Message(fp)

    attachment=0

    #check if attachments
    
    if val.getheader("Content-Type") is not None:
        ctyp=val.getheader("Content-Type")
        if string.find(ctyp, 'boundary')!= -1 or string.find(string.upper(ctyp), 'BOUNDARY')!= -1:
            attachment=1

    Exp = [None, None]
    try:
            From = val.getaddr("From")
    except:
            From = [" ", " "]


    try:
            ReplyTo=val.getheader("Reply-To")
    except:
            ReplyTo = ""

    Exp[0]=From[0]
    Exp[1]=From[1]

    try:
            if is_quoted_printable.search(From[0]):
                    Exp[0] = mimify.mime_decode_header(From[0])

            if  Exp[0] is not None and Exp[0] != "":
                    sender=Exp[0]
            elif Exp[0] is not None and Exp[0] == "":
                    sender=Exp[1]
            else:
                    sender=" Expditeur non spcifi "
    except:
            sender=" Expditeur non spcifi "

    if sender is None:
            sender=" Expditeur non spcifi "


    if is_quoted_printable.search(sender):
            sender = mimify.mime_decode_header(sender)

    try:
        too=val.getaddr("To")
    except:
        too=("", " ")
        
    if  too[0] is not None and too[0] != "":
            to=too[0]
    elif too[0] is not None and too[0] == "":
            to=too[1]
    else:
            to=" "

    suj=val.getheader("Subject")

    if suj is not None:
            subject=render_subject(suj)
    else:
            subject="[pas de sujet]"


    fp.close()


    if len(subject)>nbCharSubject:
        subject=subject[0:nbCharSubject]+'...'

    #render readable size
    if size > 1024*1024:
        size = "%.2f Mb" % (size / 1024.0 / 1024)
    else:
        size = "%.2f Kb" % (size / 1024.0)
    

    head= {}

    try:
        ddate=render_date(val.getheader("Date"))
    except:
        ddate=" format error " 

    try:
        head={'imap_id':imapId, 'sender': string.capitalize(sender[0:nbCharSubject]),'mail_sender':Exp[1], 'to': string.capitalize(to),'replyTo':ReplyTo,  'size': size, 'sujet':string.capitalize(subject), 'date':ddate, 'read':0, 'answered':0, 'deleted':0, 'flagged':0, 'attachments':attachment}
    except:
        head={'imap_id':imapId, 'sender': sender[0:nbCharSubject],'mail_sender':Exp[1], 'to': to,'replyTo':ReplyTo,  'size': size, 'sujet':subject, 'date':ddate, 'read':0, 'answered':0, 'deleted':0,'flagged':0, 'attachments':attachment}
        

    flags=string.lower(flags)

    if string.find(flags, 'seen')!=(-1):
        head["read"]=1
    if string.find(flags, 'deleted')!=(-1):
        head["deleted"]=1
    if string.find(flags, 'answered')!=(-1):
        head["answered"]=1
    if string.find(flags, 'flagged')!=(-1):
        head["flagged"]=1


    return head



def parse_RFCMessage(mess, direct_body, flags, imapid):
    """ return message instance of IMAPMessage """

    msg=cStringIO.StringIO(mess)
    message=mimetools.Message(msg)

    headers = {}
    parts = []
    body = ''
    attachments = []
    boundary=0
    body_base64=0

    boundary = message.getparam('boundary') or '' or message.getparam('BOUDDARY') or ''


    #building header keys dictionnary for the message
    for key in message.keys():

        if is_quoted_printable.search(message[key]) or is_base64.search(message[key]):
            headers[string.lower(key)] = mimify.mime_decode_header(message[key])
            
            if is_base64.search(message[key]):
                try:
                    sujet=string.replace(message[key], '=?iso-8859-1?B?', '')
                    sujet=string.replace(sujet, '=?iso-8859-1?b?', '')
                    sujet=string.replace(sujet, '=?iso-8859-2?b?', '')
                    sujet=string.replace(sujet, '=?iso-8859-2?B?', '')
                    sujet=string.replace(sujet, '?=', '')
                    encoded = StringIO.StringIO(sujet)
                    decoded = StringIO.StringIO()
                    base64.decode(encoded, decoded)
                    headers[string.lower(key)]=decoded.getvalue()
                except:
                    pass
        else:
            headers[string.lower(key)] =mimify.mime_decode(message[key])


    msg.seek(message.startofbody)

    #get and parse attachments if exist 
    if boundary:
        try:
            multi_message = multifile.MultiFile(msg)
            multi_message.push(boundary)
            liste_media=[]

            while multi_message.next():
                parsed_attachment = mime_message.mime_part_parse(multi_message)
                parts.append(parsed_attachment)
                liste_media.append(parsed_attachment.media_type)
                
            multi_message.pop()

            if parts[0].media_type == 'text':

                body = parts[0].data

                if string.find(parts[0].encoding, 'base64')!=(-1):
                    body_base64=1
                    try:
                        body=mime_decode(body, 1)
                    except:
                        pass
                parts = parts[1:]

            elif len(body) == 0 and (parts[1].media_type=='text' or parts[0].media_type=='multipart'):

                val=parts[0].data
                
                comp=re.compile('Content-Transfer-Encoding: base64', re.I)
                if comp.search(val):
                    body_base64=1

                body=re.sub(r'(------=_NextPart)([\S\s]*?)(Content-Transfer-Encoding.*)([\S\s]*?)(------=_NextPart?)([\S\s]*)', r'\4', val)

                if body_base64:
                    import base64
                    import StringIO
                    encoded = StringIO.StringIO(body)
                    decoded = StringIO.StringIO()
                    base64.decode(encoded, decoded)
                    body=decoded.getvalue()

                parts = parts[1:]

            # In any case, the rest of the parts are attachments.

            map(attachments.append, parts)
        except :
            pass
    else:
            _body = mime_message.mime_part_parse(message.fp)
            body = _body.data

    try:
        direct_body=re.sub(r'(------=_NextPart)([\S\s]*?)(Content-Transfer-Encoding.*)([\S\s]*?)(------=_NextPart?)([\S\s]*)', r'\4',direct_body)
    except:
        pass

    if body_base64:
        try:
            direct_body=mime_decode(direct_body, 1)
        except:
            pass

    if len(body) < len(direct_body):
        body=direct_body

    attachements=[]
    i=1

    for part in parts:
        id=i
        len_at=len(part.data)
        if len_at > 1024*1024:
            _size = "%.2f MB" % (len_at / 1024.0 / 1024)
        else:
            _size = "%.2f KB" % (len_at / 1024.0)
        if is_quoted_printable.search(part.filename):
            part.filename=mimify.mime_decode_header(part.filename)
        attachements.append(Attachement.Attachement(id=id,filename=part.filename,content_type=part.media_type+'/'+part.sub_type,size=_size,data=part.data))
        i=i+1
    
    _date="pas de date spcifie"
    if headers.has_key('date'):
        _date = headers['date']
        try:
            _date = render_date(_date)
        except:
            pass

    _to=[]
    if headers.has_key('to'):
        _to = headers['to']

    _cc= []
    if headers.has_key('cc'):
        _cc = headers['cc']

    _replyTo= ""
    if headers.has_key('reply-to'):
        _replyTo = headers['reply-to']

    _subject="pas de sujet"
    if headers.has_key('subject'):
        _subject = headers['subject']
        _subject = render_subject(_subject)

    _destinataires={'to': _to, 'cc': _cc, 'bcc': []}


    _fromNom = " Expditeur non spcifi "
    _fromMail = ''
    if headers.has_key('from'):
        if headers['from']:   ## J'ai un from
            if len(string.split(headers['from'],'<')) > 1: ## Je suis dans le cas 'nom < email >'
                _fromNom = string.strip(string.split(headers['from'],'<',1)[0])
                _fromMail = string.replace(string.split(headers['from'],'<',1)[1],'>','')
            else:  ## Je suis dans le cas '<email>' ou 'email'
                _fromMail = string.replace(string.replace(headers['from'],'<',''),'>','')
            if is_quoted_printable.search(_fromNom):
                try:
                    _fromNom = string.strip(mimify.mime_decode_header(_fromNom))
                except: ### Encodage inconu
                    pass

    _from = (_fromNom,_fromMail)

    dico_flags={'read':0, 'answered':0, 'deleted':0, 'flagged':0}
    flags=string.lower(flags)
    for item in ['read', 'answered', 'deleted', 'flagged']:
        if string.find(flags, item)!=(-1):
            dico_flags[item]=1

    #build IMAPMessage
    ref=IMAPMessage.IMAPMessage(IMAPId=imapid, flags=dico_flags, expediteur=_from, size=0, date=_date, subject=_subject, body=body, destinataires=_destinataires, replyTo=_replyTo, attachements=attachements, headers=headers)

    return ref


def mime_decode(line, isBase64=0):
        if isBase64: return base64.decodestring(line)
        else: return mimify.mime_decode(line)


def RFC_render_date(date):
    """Returns a formatted view of the message date."""

    try:
            rep = date
            jour = DateTime.DateTime(rep).dd()
            mois = DateTime.DateTime(rep).mm()
            annee = DateTime.DateTime(rep).yy()
            heure = DateTime.DateTime(rep).TimeMinutes()
            date = jour + '/' + mois + '/' + annee + ' ' + heure
            return date

    except KeyError:

            return '__'

    except DateTime.DateTime.DateTimeError, value:

            return 'Internal error'

    except DateTime.DateTime.SyntaxError, value:
            try:
                rep=re.sub(r'(\(.*\))', r'', date)
                jour = DateTime.DateTime(rep).dd()
                mois = DateTime.DateTime(rep).mm()
                annee = DateTime.DateTime(rep).yy()
                heure = DateTime.DateTime(rep).TimeMinutes()
                date = jour + '/' + mois + '/' + annee + ' ' + heure
                return date
            except:
                return 'date non standard'

    except Exception, value:

            return 'Unknown exception (%s)' % value

def RFC_render_subject(subject):
    """ convert coded subject to ascii"""


    if is_quoted_printable.search(subject):
        subject = mimify.mime_decode_header(subject)

    elif is_base64.search(subject):
        sujet=string.replace(subject, '=?iso-8859-1?B?', '')
        sujet=string.replace(sujet, '=?iso-8859-1?b?', '')
        sujet=string.replace(sujet, '=?iso-8859-2?b?', '')
        sujet=string.replace(sujet, '=?iso-8859-2?B?', '')
        sujet=string.replace(sujet, '?=', '')
        encoded = StringIO.StringIO(sujet)
        decoded = StringIO.StringIO()
        try:
            base64.decode(encoded, decoded)
            subject=decoded.getvalue()
        except:
            pass
    else:
        subject=mimify.mime_decode(subject)

    return subject

