######################################################################
#
# POPPasserelle = Implement a POP3 Gateway Protocol
#
# This software is governed by a license. See 
# LICENSE.txt for the terms of this license.
#
######################################################################

import timeoutsocket

from IMAPMessage import *
from SimpleCache import SimpleCache, keepAlive_POP, default_cache_timeout
import DateTime
import poplib
import string, re, time


timeoutsocket.setDefaultSocketTimeout(20) # 10 seconds de timeout sur les sockets

def create(arg):
    """ Ghost function made for compatibility """
    pass

def uid(arg1,arg2):
    """ Ghost function made for compatibility """
    return []
    pass

class POPPasserelle:
    """ POP3 gateway """

    def __init__(self, conAlive, cache_buffer = None):
        """ Contructor of a POPPasserelle"""
        # Initialze Cache Buffer for Gateway
        if cache_buffer is not None:
            self._cache = cache_buffer
        else:
            self._cache = SimpleCache(timeout = default_cache_timeout)

        self.keepAlive_POP = conAlive
            
        self._isLogged = 0
        
    def connect(self, webmail = None, server="", port=""):
        """ connet to POP3 server"""
        try:
            try:
                if not self.connexionAlive():
                    # No permanent connexion
                    need_reconnection = 1
                else:
                    # test connexion validity
                    self.connection.noop()
                    need_reconnection = 0
            except:
                try:
                    # if noop reply  but invalide state
                    # try to disconnect for clarify status
                    self.connection.quit()
                except:
                    pass
                
                need_reconnection = 1
            if need_reconnection:
                if port is not None and port != "":
                    self.connection = poplib.POP3(server,int(port))
                else:
                    self.connection = poplib.POP3(server)

                self._isLogged = 0

                if self.connexionAlive():
                    self._cache.clear()

                self.connection.create = create
                self.connection.uid = uid
        except:
            # The last chance ...
            try:
                time.sleep(3)
                if port is not None and port != "":
                    self.connection = poplib.POP3(server,int(port))
                else:
                    self.connection = poplib.POP3(server)
                self._isLogged = 0
                if self.connexionAlive():
                    self._cache.clear()
                self.connection.create = create
                self.connection.uid = uid
            except:
                if webmail is not None:
                    raise 'Redirect', webmail.absolute_url() + '/view_error_dtml'
                return 0
        return 1

    def connexionAlive(self):
        """ Return the connexion type alive or not """
        if hasattr(self, 'keepAlive_POP'):
            return self.keepAlive_POP
        return keepAlive_POP

    def _setStamp(self, stamp):
        """ set the stamp of this gateway """
        self._currentStamp = stamp

    def _getStamp(self):
        """ return the stamp of the gateaway """
        return self._currentStamp

    def login(self, login, password):
        """ login to POP3 server """
        try:
            if not self._isLogged:
                self.connection.user(login)
                self.connection.pass_(password)
                self._isLogged = 1
                #self._cache.clear()
        except:
            self._isLogged = 0
            return "LOG_FAILED"
                
    def logout(self, force = 0):
        """ logout to POP3 server """
        try:
            # if connexion is NOT permanent ... we logout
            if not self.connexionAlive() or force:
                self.connection.noop()
                self.connection.quit()
                self._isLogged = 0
                self._cache.clear()
        except:
            pass

    def listFolders(self):
        """ return the unique folder INBOX of POP3 protocol """
        return ['INBOX']
        
    def selectFolder(self, name):
        """ Select a Folder : Ignored in POP3 protocol """
        pass

    def createFolder(self, name):
        """ Create Folder : Ignored in POP3 Protocol """
        pass


    def deleteFolder(self, wmail, name):
        """ Delete a folder : Ignored in POP3 Protocol """
        pass

    def getNumberOfRecentMails(self,foldername='INBOX'):
        """ return the number of new mail """
        return 0

    def getIMAPList(self, folderName, sortmail, start = 0, how_many = "ALL"):
        """ return the sorted list of Message Id """
        # get the current number of message in the mailbox
        nb_mails = self.getNumberOfMessage(folderName)

        # Test cache buffer
        if self._cache.get('getIMAPList') and self._cache.get('currentSortMail', sortmail):
            if self._cache.get('currentNbMails', nb_mails):
                return self._cache.get('getIMAPList')
        
        def gofind(l, elem):
            """ fonction local : Retiourne l'information a partir de la strcuture de liste """
            for item in l:
                if type(item) is type([]) and len(item) > 1:
                    if item[0] == elem:
                        return item[1]
            return '0'

        def parse_date(date):
            """ return a DateTime object by a date string"""
            try:
                if string.rfind(date,':') != -1 and len(date) -1 >= string.rfind(date, ':') + 3:
                    date = date[0:string.rfind(date,':') + 3]
            except: date = ' 0'
            try:
                date = DateTime.DateTime(date)
            except: date = DateTime.DateTime()
            return date
        
        def relations(x, y, sortmail = sortmail, gofind = gofind, parse_date = parse_date):
            """ fonction local : Relation d'ordre """
            if sortmail in  ['SUBJECT','FROM']:
                return cmp(string.lower(gofind(x[1], string.lower(sortmail))), string.lower(gofind(y[1], string.lower(sortmail))))
            if sortmail == 'SIZE':
                 elem1 = int(gofind(x[1], 'size'))
                 elem2 = int(gofind(y[1], 'size'))
            else:
                # Date par defaut
                elem1 = parse_date(gofind(x[1], 'date'))
                elem2 = parse_date(gofind(y[1], 'date'))
               
            if elem1 < elem2: return 1
            elif elem1 == elem2: return 0
            else: return -1
            
        liste = []
        lt = []

        # list of size of each message
        ltai = map(lambda x:x.split()[1], self.connection.list()[1])
        for i in range(nb_mails):      
            lt = self.connection.top(i+1, 0)[1]
            # Ajoute la taille a la structure header du message
            lt.append('Size:' + ltai[i])
            # Split les liste selon ':'
            liste.append((i+1, map((lambda x:map(string.lower, string.split(x, ':', 1))), lt)))
            
  
        if string.find(sortmail, 'search') != (-1):
            #get messages list from search
            sortmail_list = string.split(sortmail, 'x2jq')
            search_where = sortmail_list[1]
            search_line = string.replace(sortmail_list[2], 'zz20', ' ')
            search_when = sortmail_list[3]
            search_date = sortmail_list[4]

            def search_in(couple,search_line=search_line,search_where=search_where,search_when=search_when,search_date=search_date,gofind=gofind,parse_date=parse_date,self=self):
                """ Local Function : search function"""
                i = 0
                l = couple[1]
                date = DateTime.DateTime(search_date).Date()
                if search_when == 'SINCE':
                    if date < parse_date(gofind(l, 'date')).Date():
                        i = i+1
                elif search_when == 'ON':
                    if date == parse_date(gofind(l, 'date')).Date():
                        i = i+1
                else:
                    if date > parse_date(gofind(l, 'date')).Date():
                        i = i+1

                if search_line:
                    p = re.compile('[\w\W\s\S]*' + search_line + '[\w\W\s\S]*', re.IGNORECASE | re.DOTALL)
                    if search_where in ['SUBJECT', 'FROM', 'TO']:
                        if p.match(gofind(l, string.lower(search_where))) is not None:
                            # Trouver
                            i = i+1
                    elif search_where == 'BODY':
                        message, direct_body, flags = self.getMessage(IMAPId = couple[0])
                        if p.match(direct_body or message) is not None:
                            i = i+1
                    else:
                        i = i+1
                else:
                    i = i+1
                return i >= 2

            # Construction de la liste filtre

            liste = filter((lambda x, search_in = search_in:search_in(x)), liste)

            
        # Tri selon sortmail 
        liste.sort(relations)
    
        # Update cache buffer
        self._cache.set('getIMAPList', liste)
        self._cache.set('currentSortMail', sortmail)
        self._cache.set('currentNbMails', nb_mails)
        
        return liste

    def listMessagesHeaders(self, folderName = "INBOX", sortmail = "", start = 0, end = 0, listing_size = 0):
        """ return headers from POP3  """
        start = int(start)
        if not end:
            end = start + int(listing_size)
        end = int(end)
        

        # Get list of message
        liste_res = self.getIMAPList(folderName=folderName, sortmail=sortmail, start=start)
        #limit the number of message in the list to number of messages by page (choose in options)
        
        liste_res = liste_res[start:end]
        liste_messages = [] 
        
        for i in range(len(liste_res)):
            rep = string.join(map(lambda x:string.join(x, ':'), liste_res[i][1][:-1]), '\r\n')    
            taille = int(liste_res[i][1][-1][1])
            IMAPId = liste_res[i][0]
            liste_messages.append({'message':rep, 'size':taille, 'flags':" ", 'imapId':IMAPId})
            
        return liste_messages


    def getFlags(self, folderName, IMAPId):
        """ return flags of a message : IGNORED POP3 """
        return " "

    def setFlag(self, folderName = "", IMAPId = "", flag = ""):
        """ set flag of a message """
        if flag == "delete":
            self.connection.dele(int(IMAPId))

            # Invalide some cached value 'cause list mesg have
            # changed
            self._cache.remove('getIMAPList')
            self._cache.remove('currentNbMails')
            #if self.connexionAlive():
                # if StandAlone Server : we close the permanent connexion
                # for commit the deletation
            #    self.connection.quit()

    def getMessage(self, folderName = "INBOX", IMAPId = ""):
        """ get full message """
        direct_body = ' '
        flags = ""
        
        mesg = self.connection.retr(int(IMAPId))[1]
        message = string.join(mesg, '\r\n')
        header = self.connection.top(int(IMAPId), 0)[1]
        direct_body2 = []
        # Enleve le header du message entier : BODY

        for item in mesg:
            if not item in header:
                direct_body2.append(item)
        direct_body = string.join(direct_body2, '\r\n')
        d2 = []

        # on enleve les pieces attaches
        #p = re.compile('------_=_NextPart_[\w]*',re.IGNORECASE)
        p = re.compile('--[\w]*', re.IGNORECASE)
        f = re.compile('Content-Type: ([\w]*/[\w]*)', re.IGNORECASE)

        parts = p.split(direct_body)
        for part in parts:
            s = f.split(part)
            if len(s) > 3:
                if s[1] in ['text/plain', 'text/html']:
                    d2.append(s[2])

        direct_body = string.join(d2, '\r\n')
                      
       	return message, direct_body, flags


    def getPreviousAndNextMessagesIds(self, folderName, sortmail, IMAPId):
        """ get next message on folder (sorted)  """

        res = map(lambda x:x[0], self.getIMAPList(folderName, sortmail))
        try:
            index = res.index(int(IMAPId))
        except:
            return None, None
        
        index_next = index + 1
        index_prev = index - 1

        prev = None
        if index_prev >= 0:
            prev = res[index_prev]

        try:
            next=res[index_next]
        except:
            next=None

        return prev, next
        

    def copy(self, folderNameSrc, folderNameDest, IMAPId):
        """ copy a message from a folder to another : POP3 => RETURN 1"""
        return 1

    def getQuota(self):
        """ return % used quota on mailbox : POP3 => Ignored """

        res = self.connection.get_quota_root()
        tab = string.split(res[0])
        return (100*int(tab[-2]))/int(string.replace(tab[-1], ")", ""))

    def getNumberOfMessage(self, folderName, search=""):
        """ return the number of messages """
        if not search:
            #number of messages in folder
            return self.connection.stat()[0]
        else:
            #number of messages from a search
            self._cache.remove('currentNbMails')
            return len(self.getIMAPList(folderName='', sortmail=search))

    def expunge(self):
        """ POP3 IGNORED expunge ! """
        return

    def writeMessage(self, folderName, raw_message):
        """ write a message : POP3 IGNORED """
        return
        
    
