
# Copyright 2001, 2002 by Fle3 Team and contributors
#
# This file is part of Fle3.


class Translation:
    """ multilanguage engine """

    def __init__(self):
        """Initialize and reload translation tables."""
        
        #import vocabulary, vocabulary_en
        self.reload_translations()

    # This is just for load speed testing.
    
    def speed_test(self,REQUEST):
        """Speed testing."""
        import time
        start=time.time()
        for i in range(5000):
            self.get_lang(REQUEST)
        stop=time.time()
        return "TIME ELAPSED: "+str(stop-start)

    # Parameters:
    #
    # - modules: a list of language modules that should be loaded
    # (by default the 'common' module)
    #
    # - REQUEST: the http request object that should receive the translations
    #
    # This method extracts the authenticated user name from the REQUEST
    # object and tries to locate that user's preferred language setting.
    # If this fails, the default language of 'en' will be used.
    #
    # The translations for the found language and requested modules
    # are then stored into the REQUEST object as key/value pairs.
    # Each keyword is prefixed by 'L_'.
    #
    # Accessing a loaded translation from DTML:
    #
    # &lt;dtml-var L_keyword&gt;
    #
    # &amp;dtml-L_keyword;
    #
    # Accessing a loaded translation from Python:
    #
    # REQUEST['L_keyword']
    
    def get_lang(self, modules=('common',), REQUEST=None):

        """Locates the language dictionary for the user and stores
        it in the REQUEST."""

        
        try:
            lang = self.get_language()
        except:
            lang='es'
        if not lang in self.langs.keys():
            lang='es'

        for module in modules:
            for (word,tuple) in self.langs[lang][module].items():
                
                REQUEST['L_'+word]=tuple[0]

    # All vocabulary*.py files are loaded from the FLE installation
    # directory. Each python file should contain a dictionary named
    # "vocabulary", which in turn contains module dictionaries which
    # in turn contain pairs of translations/descriptions by keyword.
    # See examples in vocabulary.py and vocabulary_fr.py.
    
    def reload_translations(self):
        """Reloads the translation files."""

        import vocabulary, vocabulary_en, vocabulary_es, vocabulary_ro, vocabulary_it, vocabulary_de
        
        file_path = __import__('Globals').package_home(globals())
        import os, sys
        sys.path.insert(0,file_path)
        new_langs={}
        for file in os.listdir(file_path):
            if file.find('vocabulary')==0 and \
               file.rfind('.py')==len(file)-3:
                voc_module = __import__(file[:-3])
                # If the language is marked as broken, we ignore it.
                try:
                    if voc_module.is_broken:
                        continue
                except AttributeError:
                    pass
                reload(voc_module)
                if len(file)>14:
                    lang_name=file[11:-3]
                else:
                    lang_name='fr'
                new_langs[lang_name]=voc_module.vocabulary
                
        self.langs=new_langs


    def get_languages(self):
        """Return a list of available languages, showing french first."""

        self.reload_translations()
        
        lang_list=[]
        for (name,dict) in self.langs.items():
            if name!='fr':
                lang_list.append((name,name+': '+dict['common']['language_name'][0],))
        lang_list.sort()
        lang_list.insert(
            0,
            ('fr','fr: '+self.langs['fr']['common']['language_name'][0],))
        return lang_list

