######################################################################
#
# WebMail, a IMAP/POP Webmail product for Zope
#
# This software is governed by a license. See 
# LICENSE.txt for the terms of this license.
#
######################################################################

__version__='v4.3.1'

from IMAPPasserelle import *
from POPPasserelle import *
from IMAPFolder import *
from IMAPMessage import *
from Attachement import *
from SimpleCache import SimpleCache, keepAlive_POP, keepAlive_IMAP, default_cache_timeout

import timeoutsocket, smtplib
from base64 import encodestring, decodestring

timeoutsocket.setDefaultSocketTimeout(20) # 10 seconds de timeout 

from Globals import HTMLFile
from types import *

import MessageTools

from urllib import quote, unquote
from Globals import Persistent
from DateTime import *
from OFS.SimpleItem import Item
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo
import Acquisition, AccessControl.Role, OFS, random, string

try:
    from Products.PTKBase.Document import *
except:
    pass

from Translation import *

# time out of cache : 10 min = 600 s
socketsTimeout = 600 
 
SocketsCache = SimpleCache(socketsTimeout)

def get_Socket(path):
    if SocketsCache.size() > 100:
        SocketsCache.flush()
    return SocketsCache.get(path)

def set_Socket(path, IMAPObject):
    SocketsCache.set(path, IMAPObject)

def print_Sockets():
    print SocketsCache._cache_buffer

   
def manage_add_Wmail2(self, id = 'WebMailv4_0', title = 'Wmail', REQUEST = None, RESPONSE = None):
    """Add a instance of the WebMail class"""

    try:
        wmail = WebMail(id)
        wmail.title = title
        self._setObject(id, wmail)
    except:
        id=str(random.randrange(1,999))+"WebMail"
        wmail = WebMail(id)
        wmail.title = title
        self._setObject(id, wmail)

    if REQUEST is not None:
        RESPONSE.redirect(self.absolute_url() + '/' + id + "/language_choice")


def manage_add_WebMail(self,
                       id = 'WebMailv2_0', title = '',
                       mailFrom = '', nomFrom = '',
                       login = '',password = '',
                       IMAPHost = '',IMAPPort = '',
                       SMTPHost = '', SMTPPort ='',
                       UseMailBook = "1",
                       REQUEST = None, RESPONSE = None
                       ):
    """Add a instance of the WebMail class"""
    wmail = WebMail(id)
    
    wmail.title = title
    if mailFrom:
        wmail.mailFrom = mailFrom
    if login:
        wmail.login = login
    if password:
        wmail.password = password
    if nomFrom:
        wmail.nomFrom = nomFrom
    if IMAPHost:
        wmail.IMAPHost = IMAPHost
    if IMAPPort:
        wmail.IMAPPort = IMAPPort
    if SMTPHost:
        wmail.SMTPHost = SMTPHost
    if SMTPPort:
        wmail.SMTPPort = SMTPPort
    wmail.UseMailBook = UseMailBook
    self._setObject(id, wmail)

    if REQUEST is not None:
        RESPONSE.redirect(self.absolute_url() + '/' + id)


class WebMail(Translation,Item,Persistent,
              Acquisition.Implicit,
              AccessControl.Role.RoleManager
              ):
    """ WebMail object """

    meta_type = 'WebMail'

    security = ClassSecurityInfo()

    manage = manage_main = HTMLFile('dtml/WebMailManage', globals())
    manage_main._setName('manage_main')
    
    manage_options=(
        (
        {'icon':'', 'label':'Edit',
         'action':'manage_main', 'target':'manage_main',
         'help':('','')},
        {'icon':'', 'label':'View',
         'action':'index_html', 'target':'manage_main',
         'help':('','')}
        )
        +AccessControl.Role.RoleManager.manage_options
        +OFS.SimpleItem.Item.manage_options
        )

    localHeader      = HTMLFile('dtml/standard_html_header', globals())
    localFooter      = HTMLFile('dtml/standard_html_footer', globals())

    css_style_header = HTMLFile('style_sheet/css_style_header', globals())

    ##################################################
    #### Style Sheets
    ##################################################
    
    wMailStyle1      = HTMLFile('style_sheet/webMailStyle1',  globals())
    wMailStyle2      = HTMLFile('style_sheet/webMailStyle2',  globals())
    wMailStyle3      = HTMLFile('style_sheet/webMailStyle3',  globals())
    wMailStyle4      = HTMLFile('style_sheet/iceandsunstyle', globals())
    wMailStyle5      = HTMLFile('style_sheet/icestyle',       globals())
    wMailStyle6      = HTMLFile('style_sheet/sunstyle',       globals())

    ##################################################
    #### DTML pages
    ##################################################

    security.declareProtected('View','',
                              'language_choice_dtml',
                              'view_folder',
                              'view_message_dtml',
                              'edit_message_dtml',
                              'add_attachement',
                              'editOptionForm',
                              'editConfiguration_dtml',
                              'searchForm',
                              'confirme_delete_folder',
                              'create_folder',
                              'view_printable_message_dtml',
                              'view_message_source_dtml',
                              'view_error_dtml',
                              'createAddressBook',
                              'mailBook_dtml',
                              'insertFile_html',
                              )  
    language_choice_dtml        = HTMLFile('dtml/first_step_install',     globals())
    view_folder                 = HTMLFile('dtml/view_folder',            globals())
    view_message_dtml           = HTMLFile('dtml/view_message',           globals())
    edit_message_dtml           = HTMLFile('dtml/edit_message',           globals())
    add_attachement             = HTMLFile('dtml/add_attachement',        globals())
    editOptionForm              = HTMLFile('dtml/editOption',             globals())
    editConfiguration_dtml      = HTMLFile('dtml/editConfiguration',      globals())
    searchForm                  = HTMLFile('dtml/search',                 globals())
    confirme_delete_folder      = HTMLFile('dtml/confirme_delete_folder', globals())
    create_folder               = HTMLFile('dtml/create_folder',          globals())
    view_printable_message_dtml = HTMLFile('dtml/view_printable_message', globals())
    view_message_source_dtml    = HTMLFile('dtml/view_message_source',    globals())
    view_error_dtml             = HTMLFile('dtml/view_error',             globals())
    createAddressBook          = HTMLFile('dtml/createAddressBook',      globals())
    mailBook_dtml               = HTMLFile('dtml/mailBook',               globals())
    insertFile_html             = HTMLFile('dtml/insertFile_html',        globals())

    ##################################################
    ##### Pics
    ##################################################

    icon                 = 'misc_/WebMail/icon_mail.gif'
    image_folder         = 'misc_/WebMail/folder.gif'
    image_flag           = 'misc_/WebMail/flag.gif'
    image_composer       = 'misc_/WebMail/composer.gif'
    image_messageLu      = 'misc_/WebMail/messageLu.gif'
    image_messageRep     = 'misc_/WebMail/messageRep.gif'
    image_retirer        = 'misc_/WebMail/retirer.gif'
    image_supprimer      = 'misc_/WebMail/supprimer.gif'
    image_creerDossier   = 'misc_/WebMail/creerDossier.gif'
    image_messageNonLu   = 'misc_/WebMail/messageNonLu.gif'
    image_options        = 'misc_/WebMail/options.gif'
    image_soulignmail    = 'misc_/WebMail/soulignmail.gif'
    image_wmaillogo      = 'misc_/WebMail/wmaillogo.gif'
    image_trombone       = 'misc_/WebMail/trombone.gif'
    image_cacheTrombone  = 'misc_/WebMail/cacheTrombone.gif'
    image_repondre       = 'misc_/WebMail/repondre.gif'
    image_repondreTous   = 'misc_/WebMail/repondreTous.gif'
    image_faireSuivre    = 'misc_/WebMail/faireSuivre.gif'
    image_messageRep     = 'misc_/WebMail/messageRep.gif'
    image_imprimanteMail = 'misc_/WebMail/imprimanteMail.gif'
    image_composerPetit  = 'misc_/WebMail/composerPetit.gif'
    image_poubelle       = 'misc_/WebMail/poubelle.gif'

    use_webmail_cookie = 0


    def __init__(self, id, signature="", autoSaveSentMessage=1, nbCharSubject=25,
                 listingSize=15, sortmail="order",mailFrom="",
                 IMAPHost='', IMAPPort=0, SMTPHost='', SMTPPort=0,
                 mailBook=[], wmailLang="fr", wmailtheme="theme4",
                 defaultFoldersNames = {},mailFormat = 'text/plain'):
        """ constructor """

        self.id = id
        self.signature = signature
        self.autoSaveSentMessage = autoSaveSentMessage
        self.nbCharSubject = nbCharSubject
        self.listingSize = listingSize
        self.sortmail = sortmail
        self.mailBook = ["Test MailBook,ticeuds@chez.com", "Test MailBook2,ticeuds@chez.com"]
        self.wmailtheme = wmailtheme
        self.wmailLang = wmailLang
        #init multilanguage engine
        self.reload_translations()
        self.mailFormat = mailFormat
       
    security.declareProtected('View','getIcon')  
    def getIcon(self, icon_name):
        """ return the icon """
        ic = getattr(self, icon_name, None)

        if ic is None:
            return ''
        
        if hasattr(self,'getPortal'):
            return self.getPortal().absolute_url() + '/images/Products/WebMail/icons/' + string.replace(ic, 'misc_/WebMail/','')
        else:
            return ic
        
    security.declareProtected('View','icon_html')  
    def icon_html(self, width=16, height=16):
        """ return the icon of the webmail for interface """
        return "<IMG SRC='" + self.getIcon('icon') + "' BORDER='0' width='%d' height='%d'" % (width, height)

    security.declareProtected('View','update_params')  
    def update_params(self, REQUEST, con = None):
        """ update connection and login params """
        
        if REQUEST.mailFrom != self.getMailFrom():
            self.mailFrom = REQUEST.mailFrom
        if REQUEST.replyTo != self.getReplyTo():
            self.replyTo = REQUEST.replyTo
        if REQUEST.login != self.getIMAPLogin():
            self.login = REQUEST.login
        if self.use_webmail_cookie == 0:
            if REQUEST.password != self.getIMAPPassword() :
                self.password = REQUEST.password
        else:
            if REQUEST.has_key('password'):
                REQUEST.set('imap_login', self.login)
                REQUEST.set('imap_password', REQUEST.password)
                self.setIMAPCookie(REQUEST, REQUEST.RESPONSE)

        # Traitement des nouveau parametres IMAP folder
        new_dico = {}
        new_dico['INBOX'] = [REQUEST['inbox_value'], REQUEST['inbox_label']]
        new_dico['draft'] = [REQUEST['draft_value'], REQUEST['draft_label']]
        new_dico['trash'] = [REQUEST['trash_value'], REQUEST['trash_label']]
        new_dico['sent-mail'] = [REQUEST['sentmail_value'], REQUEST['sentmail_label']]
        if new_dico != self.getDefaultFoldersNames():
            self.defaultFoldersNames = new_dico
        
        if REQUEST.nomFrom != self.getNomFrom():
            self.nomFrom = REQUEST.nomFrom
        if REQUEST.IMAPHost != self.getIMAPServer():
            self.IMAPHost = REQUEST.IMAPHost
        if REQUEST.IMAPPort != self.getIMAPPort():
            self.IMAPPort = REQUEST.IMAPPort
        if REQUEST.SMTPHost != self.getSMTPServer():
            self.SMTPHost = REQUEST.SMTPHost
        if REQUEST.SMTPPort != self.getSMTPPort():
            self.SMTPPort = REQUEST.SMTPPort
        if REQUEST.wmail_title != self.title:
            self.title = REQUEST.wmail_title
        if REQUEST.protocol != self.getProtocol():
            self.setProtocol(REQUEST.protocol)
        if REQUEST.has_key('keepAlive'):
            self.setConnexionAlive(1)
        else:
            self.setConnexionAlive(0)

        # generate a Stamp for multi thread (ZEO) and keepAlives connexion
        self._upStamp()

        try:
            #test new params
            if con is None:
                con = self.getPasserelle()
                con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
                if con.login(self.getIMAPLogin(), self.getIMAPPassword()) == "LOG_FAILED":
                    return REQUEST.response.redirect(self.absolute_url() + "/editConfiguration_dtml?params_error=1")
                con.logout()        
            try:
                self.getIMAPFolders(con = con)
            except:
                pass

            smtp_connection = smtplib.SMTP(self.getSMTPServer())
            _recipients = []
            _recipients.append(self.getMailFrom())
            raw_message = "From: webmail_installer <ticeus@chez.com>\nSubject:Test Mail\nTo:" + \
                         self.getMailFrom() + "\n\n\n this is an automatic params test configuration"
            smtp_connection.sendmail(self.getMailFrom(), _recipients, raw_message)
            smtp_connection.quit()

            return REQUEST.response.redirect(self.absolute_url() + "/index_html")
        except:

            return REQUEST.response.redirect(self.absolute_url() + "/editConfiguration_dtml?params_error=1")

    security.declareProtected('View','resetConfiguration')  
    def resetConfiguration(self, REQUEST):
        """
        del user change in configuration
        """
        from Acquisition import aq_base
        _self = aq_base(self)
        if hasattr(_self, 'protocol'):
            delattr(_self, 'protocol')
        if hasattr(_self, 'login'):
            delattr(_self, 'login')
        if hasattr(_self, 'password'):
            delattr(_self, 'password')
        if hasattr(_self, 'mailFrom'):
            delattr(_self, 'mailFrom')
        if hasattr(_self, 'mailFrom'):
            delattr(_self, 'mailFrom')
        if hasattr(_self, 'replyTo'):
            delattr(_self, 'replyTo')
        if hasattr(_self, 'IMAPHost'):
            delattr(_self, 'IMAPHost')
        if hasattr(_self, 'IMAPPort'):
            delattr(_self, 'IMAPPort')
        if hasattr(_self, 'SMTPHost'):
            delattr(_self, 'SMTPHost')
        if hasattr(_self, 'SMTPPort'):
            delattr(_self, 'SMTPPort')
        if hasattr(_self, '_cache_timeout'):
            delattr(_self, '_cache_timeout')
        if hasattr(_self, 'keepAlive'):
            delattr(_self, 'keepAlive')
        if hasattr(_self, '_v_connection'):
            delattr(_self, '_v_connection')
        self.defaultFoldersNames = {}

        # generate a Stamp for multi thread (ZEO) and keepAlives connexion
        self._upStamp()            

        return REQUEST.RESPONSE.redirect(self.absolute_url())

    default_protocol = 'IMAP'

    security.declareProtected('View','getProtocol')  
    def getProtocol(self):
        """
        return the protocol of this webmail : by default attribut => IMAP
        this is for compatibility
        """
        if not hasattr(self, 'protocol'):
            return self.default_protocol
        else:
            return self.protocol

    security.declareProtected('View','setProtocol')  
    def setProtocol(self, protocol):
        """ Set the value of the protocol (POP3/IMAP)"""
        self.protocol = protocol

    security.declareProtected('View','_upStamp')  
    def _upStamp(self):
        """ set the stamp for the current passerelle """
        if not hasattr(self, '_currentPasserelleStamp'):
            self._currentPasserelleStamp = 0
        self._currentPasserelleStamp += 1

    security.declareProtected('View','_getStamp')  
    def _getStamp(self):
        """ return the number of the current stamp for passerelle """
        if not hasattr(self, '_currentPasserelleStamp'):
            self._currentPasserelleStamp = 0
        return self._currentPasserelleStamp

    security.declareProtected('View','getPasserelle')  
    def getPasserelle(self):
        """ return the GateAway object of this webmail """
        if not hasattr(self,'_v_cache_buffer'):
            self._v_cache_buffer = SimpleCache(timeout = self.getCacheTimeout())
        elif self._v_cache_buffer.getTimeout() != self.getCacheTimeout():
            self._v_cache_buffer.setTimeout(self.getCacheTimeout())

        conAlive = self.connexionAlive()
        if not hasattr(self, '_currentConAlive'):
            self._currentConAlive = conAlive
            if hasattr(self, '_v_connection'):
                delattr(self, '_v_connection')
        if hasattr(self, '_v_connection') and self._v_connection and self._currentConAlive == conAlive \
               and self._v_connection._getStamp() == self._getStamp():
            return self._v_connection
        if self._currentConAlive != conAlive:
            self._currentConAlive = conAlive
        
        if self.getProtocol() == 'POP3' and not conAlive:
            # Create POP Object at each time 'cause Connexion is Simple (not Alive)
            return POPPasserelle.POPPasserelle(conAlive, self._v_cache_buffer)
        elif self.getProtocol() == 'POP3':
            # The Connexion POP is Alive : Try to find an initialized
            # sockets in the cache buffer of sockets
            if get_Socket(self.absolute_url(1)) is not None:
                self._v_connection = get_Socket(self.absolute_url(1))
            else:
                # we didn't find socket for this object :
                # build it and store in the cache buffer of sockets
                self._v_connection = POPPasserelle.POPPasserelle(conAlive, self._v_cache_buffer)
                set_Socket(self.absolute_url(1), self._v_connection)
        elif conAlive:
            # The Connexion IMAP is Alive : Try to find an initialized
            # sockets in the cache buffer of sockets
            if get_Socket(self.absolute_url(1)) is not None:
                self._v_connection = get_Socket(self.absolute_url(1))
            else:
                # Not Find Socket : build it and store it in cache buffer sockets
                self._v_connection = IMAPPasserelle.IMAPPasserelle(conAlive, self._v_cache_buffer)
                set_Socket(self.absolute_url(1), self._v_connection)
        else:
            # The Connexion is Not Alive : build IMAP Object at each time
            return IMAPPasserelle.IMAPPasserelle(conAlive, self._v_cache_buffer)

        self._v_connection._setStamp(self._getStamp())

        return self._v_connection

    security.declareProtected('View','isUsingCookie')  
    def isUsingCookie(self):
        """ """
        return self.use_webmail_cookie


    security.declareProtected('View','encodage')  
    def encodage(self, t):
        """ """
        return string.replace(encodestring(t), '\n', '')

    security.declareProtected('View','setIMAPCookie')  
    def setIMAPCookie(self, REQUEST, RESPONSE, con = None):
        """ set a cookie for imap authentification param """
        if REQUEST.has_key('imap_login'):
            log = REQUEST['imap_login']
        if REQUEST.has_key('imap_password'):
            pas = REQUEST['imap_password']
        if log and pas and con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            l = con.login(log,pas)
            con.logout()
            if l == "LOG_FAILED":
                return RESPONSE.redirect(self.absolute_url() + '/view_error_dtml?warning=1')
            
            key = self.encodage(log + ':' + pas)
	    if hasattr(self, 'use_AES') and self.use_AES:
                from urllib import quote
                RESPONSE.setCookie( 'imap_ac', quote(self.encrypt_cookie(key)))
            else:
                RESPONSE.setCookie('imap_ac', key)
        return RESPONSE.redirect(self.absolute_url() + '/portal_contents')

    security.declarePrivate('_getAESKey')
    def _getAESKey(self):
        current_date = DateTime.DateTime().millis()
        lastkey = getattr(self, '_AES_KEY', None)
        if lastkey is None:
            new_key = self.gen_key() # 32 bytes long
            self._AES_KEY=(new_key, current_date)
            return new_key
        else:
            if current_date - lastkey[1] > 86400000:
                new_key = self.gen_key() # 32 bytes long
                self._AES_KEY=(new_key, current_date)
                return new_key
            else:
                return lastkey[0]
    
    # encode en AES la phrase "value" avec la cl AES_key
    security.declareProtected('View','encrypt_cookie')  
    def encrypt_cookie(self, value):
        """ """
	from Crypto.Cipher import AES
        if len(value) > 16 :
            nb_car = 16 - (len(value) % 16)
        else:
            nb_car = 16 - len(value)

        # Il faut que la longueur de phrase  encoder soit un multiple de 16
        if nb_car != 0:
            for i in range(0, nb_car):
                value = value + chr(nb_car)

        cle = AES.new(self._getAESKey(), AES.MODE_ECB)

        return cle.encrypt(value)

    # decode une phrase code en AES
    security.declareProtected('View','decrypt_cookie')  
    def decrypt_cookie(self, value):
        """ """
	from Crypto.Cipher import AES
        cle = AES.new(self._getAESKey(), AES.MODE_ECB)
        val_decode = cle.decrypt(value)
        last_car = val_decode[-1:]

        if ord(last_car) in range(1,16):  # on retire les caractres en trop ajouts  l'encodage
            last_car = int(ord(last_car))
            for i in val_decode[0-last_car:]:
                if int(ord(i)) != last_car:
                    return val_decode

            return val_decode[:0-last_car]

        return val_decode
    # SP mod -- end

    security.declareProtected('View','getIMAPLogin')  
    def getIMAPLogin(self): 
        """ return IMAP login """
        if self.use_webmail_cookie == 1:
            REQUEST = self.REQUEST
            RESPONSE = REQUEST.RESPONSE
            if REQUEST.has_key('imap_ac'):
	        from urllib import unquote
                if hasattr(self, 'use_AES') and self.use_AES:
                   return  string.split(decodestring(self.decrypt_cookie(unquote(REQUEST['imap_ac']))), ':', 1)[0]
                else:
                   return string.split(decodestring(REQUEST['imap_ac']),':', 1)[0]
            else:
                raise  'Redirect', self.absolute_url() + '/view_error_dtml?use_cookie=1'
        try:
            if hasattr(self, 'login'):
                return self.login
            elif self.getAuthenticatedMemberLDAPData(self.mail_login_LDAP_attribut):
                return self.getAuthenticatedMemberLDAPData(self.mail_login_LDAP_attribut)
            else:
                return self.getAuthenticatedMemberName()
        except:
            return ''

    security.declareProtected('View','getIMAPPassword')  
    def getIMAPPassword(self):
        """ return IMAPPassword """
        if self.use_webmail_cookie == 1:
            REQUEST = self.REQUEST
            RESPONSE = REQUEST.RESPONSE
            if REQUEST.has_key('imap_ac'):
                from urllib import unquote
                if hasattr(self, 'use_AES') and self.use_AES:
                   return  string.split(decodestring(self.decrypt_cookie(unquote(REQUEST['imap_ac']))), ':', 1)[1]
                else:
                   return string.split(decodestring(REQUEST['imap_ac']), ':', 1)[1]
            else:
                raise  'Redirect', self.absolute_url() + '/view_error_dtml?use_cookie=1'

        try:
            if hasattr(self, 'password'):
                return self.password
            
            mailPasswordLdapAttr = self.getAuthenticatedMemberLDAPData(self.mail_password_LDAP_attribut)

            if not mailPasswordLdapAttr:
                mailPasswordLdapAttr = self.getAuthenticatedMemberPassword()

            return mailPasswordLdapAttr
        except:
            return ''

    security.declareProtected('View','getReplyTo')  
    def getReplyTo(self):
        """ return reply to """
        try:
            if hasattr(self, 'replyTo'):
                return self.replyTo
            else:
                mailFrom = self.getMailFrom()
                if type(mailForm) is type([]):
                    mailFrom = mailFrom[0]
                return mailFrom
        except:
            return ''

    security.declareProtected('View','getMailFrom')  
    def getMailFrom(self):
        """ return mail from """
        try:
            if hasattr(self, 'mailFrom'):
                return self.mailFrom
            
            mailFromLdapAttr = self.getAuthenticatedMemberLDAPData(self.mail_from_LDAP_attribut)
            if not mailFromLdapAttr:
                mailFromLdapAttr = self.getAuthenticatedMemberLDAPData(self.mailLDAPAttribut)
            if type(mailFromLdapAttr) is type([]):
                    mailFromLdapAttr = mailFromLdapAttr[0]

            return mailFromLdapAttr
        except:
            return ''


    security.declareProtected('View','getNomFrom')  
    def getNomFrom(self):
        """ return nom from """
        try:
            if hasattr(self, 'nomFrom'):
                return self.nomFrom
            else:
                return self.getAuthenticatedMemberLDAPData('cn')
        except:
            return ''

    security.declareProtected('View','getMailFormat')  
    def getMailFormat(self, mf=None):
        """ retrun the mailformat choose by the user """
        if mf is not None:
            return mf
        if not hasattr(self, 'mailFormat'):
            self.mailFormat = 'text/plain'
        return self.mailFormat
    
    security.declareProtected('View','getDomain')  
    def getDomain(self): 
        """ return the name of the amil server domaine -ie all after @ """ 
        return self.mail_domain 

    security.declareProtected('View','getCacheTimeout')  
    def getCacheTimeout(self):
        """ Retourne the value of the cache timeout"""
        if hasattr(self,'_cache_timeout'):
            return self._cache_timeout
        elif hasattr(self, 'mail_cache_timeout'):
            return self.mail_cache_timeout
        else:
            return default_cache_timeout

    security.declareProtected('View','setCacheTimeout')  
    def setCacheTimeout(self, timeout):
        """ Set the value of the cache timeout"""
        self._cache_timeout = timeout
        if hasattr(self,'_v_cache_buffer') and self._v_cache_buffer:
            self._v_cache_buffer.setTimeout(timeout)

    security.declareProtected('View','setConnexionAlive')  
    def setConnexionAlive(self, flag):
        """ Set the connexion type """
        if not hasattr(self, 'keepAlive') or self.keepAlive != flag:
            self.keepAlive = flag

    security.declareProtected('View','connexionAlive')  
    def connexionAlive(self):
        """ Return if the connexion is alive or not """
        
        if hasattr(self, 'keepAlive'):
            ret = self.keepAlive
        elif self.getProtocol() == 'POP3' and hasattr(self, 'mail_keepAlive_POP'):
            ret = self.mail_keepAlive_POP
        elif self.getProtocol() == 'POP3':
            ret = keepAlive_POP
        elif hasattr(self, 'mail_keepAlive_IMAP'):
            ret = self.mail_keepAlive_IMAP
        else:
            ret = keepAlive_IMAP
        if not ret and hasattr(self, '_v_connection'):
            self._v_connection = None
        return ret

    security.declareProtected('View','getIMAPServer')  
    def getIMAPServer(self):
        """ return the IMAP server name"""
        try:
            if hasattr(self, 'IMAPHost'):
                return self.IMAPHost
            elif hasattr(self, 'mail_server_LDAP_attribut_eval') and self.mail_server_LDAP_attribut_eval:
                result = self.Execute(self.mail_server_LDAP_attribut_eval, 1)
                if result and type(result) is type([]):
                    result = result[0]
                return result
            elif self.getAuthenticatedMemberLDAPData(self.mail_server_LDAP_attribut):
                return self.getAuthenticatedMemberLDAPData(self.mail_server_LDAP_attribut)
            else:
                return self.default_mail_server
        except:
            return ''
        
    security.declareProtected('View','getIMAPPort')    
    def getIMAPPort(self):
        """ return the IMAP server port """
        try:
            if hasattr(self, 'IMAPPort'):
                return self.IMAPPort
            else:
                return ''
        except:
            return ''

    security.declareProtected('View','getSMTPServer')  
    def getSMTPServer(self):
        """ return the SMTP server name"""
        try:
            if hasattr(self, 'smtp_server_LDAP_attribut_eval') and self.smtp_server_LDAP_attribut_eval:
                result = self.Execute(self.smtp_server_LDAP_attribut_eval, 1)
                if result and type(result) is type([]):
                    result = result[0]
                return result
            elif self.getAuthenticatedMemberLDAPData(self.smtp_server_LDAP_attribut):
                return self.getAuthenticatedMemberLDAPData(self.smtp_server_LDAP_attribut)
            else:
                return self.default_smtp_server
        except:
            if hasattr(self, 'SMTPHost'):
                return self.SMTPHost
            else:
                return ''

    security.declareProtected('View','getSMTPPort')  
    def getSMTPPort(self):
        """ return the SMTP server port """

        if hasattr(self, 'SMTPPort'):
            return self.SMTPPort
        else:
            return None

    security.declareProtected('View','getVersion')  
    def getVersion(self):
        """ return the product version"""

        return __version__

    security.declareProtected('View','getDefaultFoldersNames')  
    def getDefaultFoldersNames(self):
        """ return the good dico for the webmail """
        if hasattr(self, 'defaultFoldersNames') and self.defaultFoldersNames != {}:
            return self.defaultFoldersNames
        self.setDefaultFoldersNames()
        return self.defaultFoldersNames
             
    security.declareProtected('View','index_html')  
    def index_html(self, IMAPName="", start=0, REQUEST=None, con = None):
        """ return default page """
        if self.getIMAPServer() == '':
            raise 'Redirect', self.absolute_url() + '/view_error_dtml'
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())

        if IMAPName:
            folder = self.getFolder(IMAPName, con)
        else:
            folder = self.getInboxFolder()

        ret = self.view_folder(self, folder = folder, start = start, REQUEST = REQUEST, con = con)
        con.logout()
        return ret

    security.declareProtected('View','portal_contents')  
    portal_contents = index_html

    security.declareProtected('View','language_choice')  
    def language_choice(self):
        """ return the language choice page """

        return self.language_choice_dtml(self)

    security.declareProtected('View','editOption')  
    def editOption(self, REQUEST, con = None):
        """ Return the option of the webmail """
        if con is None and self.getProtocol() != 'POP3':
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        ret = self.editOptionForm(self, REQUEST, con = con)
        if con is not None:
            con.logout()
        return ret

    security.declareProtected('View','editConfiguration')  
    def editConfiguration(self):
        """ return params edit page """

        return self.editConfiguration_dtml(self, self.REQUEST)

    security.declareProtected('View','view_error')  
    def view_error(self, REQUEST = None, RESPONSE = None):
        """ return error page """
        return self.view_error_dtml(self, REQUEST = REQUEST, RESPONSE = RESPONSE)

    security.declareProtected('View','viewMailBook')  
    def viewMailBook(self):
        """ return mailBook page """
        return self.mailBook_dtml(self)

    security.declareProtected('View','adPeopleMailBook')  
    def adPeopleMailBook(self, REQUEST = None, new_contact = None):
        """ delete peolple on MailBook """
        
        try:
            if new_contact is None and (len(REQUEST.mailBookMail) > 2):
                contact = string.replace(REQUEST.mailBookName, ",", " ")+"," + string.replace(REQUEST.mailBookMail, ",", " ")
            else:
                contact = new_contact
            if contact is not None:
                contact = string.capwords(string.replace(contact, '"', ''))
                tempList = self.mailBook
                tempList.append(contact)
                self.mailBook=tempList
        except:
            pass

        if new_contact is None:
            return self.mailBook_dtml(self)

    security.declareProtected('View','delPeopleMailBook')  
    def delPeopleMailBook(self, REQUEST):
        """ delete peolple on MailBook """

        try:
            tempList = self.mailBook
            del tempList[tempList.index(unquote(REQUEST.peopleToDelete))]
            self.mailBook = tempList
        except:
            pass
        
        return self.mailBook_dtml(self)
    
    security.declareProtected('View','view_message')  
    def view_message(self, IMAPName = "INBOX", IMAPId = "", REQUEST = None, con = None):
        """ return the view interface of the message """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        try:
            folder = self.getFolder(IMAPName, con)
            if not folder:
                folder = self.getFolder('inbox', con)
                IMAPName = "inbox"
            message = folder.getIMAPMessage(IMAPId, con)
        except:
            return self.view_error()
        
        if message != "DELETED":
            ret = self.view_message_dtml(self, message = message, IMAPName = IMAPName, REQUEST = REQUEST, con = con)
        else:
            ret = "erreur de retrait du message"
        con.logout()
        return ret

    security.declareProtected('View','view_message_source')  
    def view_message_source(self, IMAPName = "INBOX", IMAPId = "", REQUEST = None, con = None):
        """ return the view interface of the message """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())

        folder = self.getFolder(IMAPName, con)
        
        message = folder.getRawIMAPMessage(IMAPId, con)
        
        if message != "DELETED":
            ret = self.view_message_source_dtml(self, message = message, IMAPName = IMAPName, REQUEST = REQUEST, con = con)
        else:
            ret = "erreur de retrait du message"
        con.logout()
        return ret

    security.declareProtected('View','setMessageFlagged')  
    def setMessageFlagged(self, IMAPName = "INBOX", IMAPId = "", REQUEST = None, con = None):
        """ set on the Flagged IMAP flag """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        try:
            if REQUEST.isFlagged == "1":
                con.setFlag(folderName = IMAPName, IMAPId = IMAPId, flag = "-flagged")
            else:
                con.setFlag(folderName = IMAPName, IMAPId = IMAPId, flag = "+flagged")
        except:
            pass

        try:
            folder = self.getFolder(IMAPName, con)
            message = folder.getIMAPMessage(IMAPId, con)
        except:
            if con is not None:
                con.logout()
            return self.view_error()
        
        if message != "DELETED":
            ret = self.view_message_dtml(self, message = message, IMAPName = IMAPName, REQUEST = REQUEST, con = con)
        else:
            ret = "erreur de retrait du message"
        con.logout()
        return ret

    security.declareProtected('View','view_printable_message')  
    def view_printable_message(self, IMAPName = "INBOX", IMAPId = "", REQUEST = None, con = None):
        """ return the view interface of the message """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        folder = self.getFolder(IMAPName, con)
        message = folder.getIMAPMessage(IMAPId, con)
        if message != "DELETED":
            ret = self.view_printable_message_dtml(self, message = message, IMAPName = IMAPName, REQUEST = REQUEST, con = con)
        else:
            ret = "erreur de retrait du message"
        con.logout()
        return ret

    security.declareProtected('View','edit_message')  
    def edit_message(self, IMAPName = "INBOX", IMAPId = "",subject = "", body = "", destinataires = {'to':[], 'cc':[], 'bcc':[]}, attachements = [], REQUEST = None, con = None):
        """ return the edit interface on this message """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        folder = self.getFolder(IMAPName, con)
        if IMAPId:
            #edit on forward or reply message
            message = folder.getIMAPMessage(IMAPId, con)
        else:
            if REQUEST.has_key('mailto'):
                if len(string.split(REQUEST['mailto'], ',')) == 2:
                    mailto = string.split(REQUEST['mailto'], ',')
                else:
                    mailto = [REQUEST['mailto']]
                destinataire = {'to':mailto, 'cc':[], 'bcc':[]}

            #edition new message
            message = IMAPMessage(IMAPId = "", flags = [], expediteur = [], \
                                  size = 0, date = "", subject = subject, body = body, \
                                  destinataires = destinataires, attachements = attachements, headers = {})
            
        ret = self.edit_message_dtml(self, message = message, IMAPName = IMAPName, REQUEST = REQUEST, con = con)
        con.logout()
        return ret

    security.declareProtected('View','createIMAPFolder')  
    def createIMAPFolder(self, REQUEST, con = None):
        """ create a new folder on IMAP server """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        res_create = con.createFolder(REQUEST.title)
        con.logout()

        IMAPName = REQUEST.IMAPName

        if res_create != "NO":
            #creation sucessfull
            return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' + quote(REQUEST.IMAPName))
        else:
            #creation unsucessfull
            return REQUEST.response.redirect(self.absolute_url() + '/create_folder?IMAPName=' + quote(REQUEST.IMAPName) +'&error_name=1')

    security.declareProtected('View','deleteIMAPFolders')  
    def deleteIMAPFolders(self, REQUEST = None, con = None):
        """ delete a IMAP folder """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        for folder in REQUEST.IMAPNamesToDelete:
            res_del = con.deleteFolder(wmail = self, name = folder)
        con.logout()
        IMAPName = REQUEST.IMAPName
        return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' + quote(REQUEST.IMAPName))

    security.declareProtected('View','emptyTrash')  
    def emptyTrash(self, REQUEST=None, con = None):
        """ empty trash """

        IMAPName = REQUEST.IMAPName
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        con.selectFolder(self.getTrashFolder().getImapName())
        res = con.connection.uid('SEARCH', 'ALL')
        if res != []:
            liste_res = map(int, string.split(res[-1][-1]))
            for IMAPId in liste_res:
                con.setFlag(IMAPId = IMAPId, flag = "delete")
        con.expunge()        
        con.logout()
        return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' + quote(IMAPName))

    security.declareProtected('View','getNumberOfRecent')  
    def getNumberOfRecent(self, foldername = 'INBOX', con  = None):
        """ return the number of recent mail"""
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        return con.getNumberOfRecentMails(foldername)

    security.declareProtected('View','reply')  
    def reply(self, REQUEST = None, con = None):
        """ render reply formated message """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
            
        folder = self.getFolder(REQUEST.IMAPName, con)
        message = folder.getIMAPMessage(REQUEST.IMAPId, con)

        message.setSubject('Re: ' + message.getSubject())
        if EMAIL_LIB:
            message.setBody(message.render_body(message.getBody(), 'text', reply = 1))
        else:
            message.setBody('\n> ' + string.join(string.split(message.render_body(message.getBody(), 'html'),'\n'), '\n> ') + '\n\n')
            
        all = int(REQUEST.all)

        if not all:
            #reply to sender or REPLYTO!!!!!!!!!!!!!!!
            if message.getReplyTo() != '':
                message.setTo(message.getReplyTo())
            else:
                message.setTo(message.getExpediteurMail())
            message.setCC([])
        else:
            #reply all
            if message.getReplyTo() != '':
                recipients = message.getReplyTo() + ', ' + message.getTo()
            else:
                recipients = message.getExpediteurMail() + ', ' + message.getTo()
            message.setTo(recipients)

        message.attachements = []

        ret = self.edit_message_dtml(self, message=message, attReply='yes', redirection_url='reply', \
                                     IMAPName=REQUEST.IMAPName, REQUEST=REQUEST, con = con)
        con.logout()
        return ret
        
    security.declareProtected('View','forward')  
    def forward(self, REQUEST, con = None):
        """ render forward message """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
            
        folder = self.getFolder(REQUEST.IMAPName, con)
        message = folder.getIMAPMessage(REQUEST.IMAPId, con)
        
        message.setSubject('Fwd: ' + message.getSubject())
        message.setDestinataires({'to':[], 'cc':[], 'bcc':[]})

        body = "\n\n> Expditeur d'origine: "
        body = body + message.getExpediteurName() + ' (' + message.getExpediteurMail() + ') '
        body = body + "\n> Date: " + MessageTools.render_date(message.getDate()) + "\n\n"

        if EMAIL_LIB:
            new_body = message.render_body(message.getBody(), 'text', reply = 1)
        else:
            new_body = string.split(message.render_body(message.getBody(), 'html'), '\n')
            new_body = '> ' + string.join(new_body, '\n> ')  + ' \n\n'
            
        new_body = body + new_body
        message.setBody(new_body)

        ret = self.edit_message_dtml(self, message=message, IMAPName=REQUEST.IMAPName, REQUEST=REQUEST, attForward = 'yes', \
                                     redirection_url='forward', con = con)
        con.logout()
        return ret

    security.declareProtected('View','search')  
    def search(self, REQUEST, con = None):
        """ return the search interface """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        ret = self.searchForm(self, REQUEST, con = con)
        con.logout()
        return ret

    security.declareProtected('View','searchMail')  
    def searchMail(self, REQUEST=None):
        """ send search request on imap server """

        flagged = "non"
        if REQUEST.has_key('only_flagged'):
            if REQUEST.only_flagged == "oui":
                flagged = "oui"

        search_date = REQUEST.search_day + "-" + REQUEST.search_month + "-" + REQUEST.search_year
        search_line = string.replace(REQUEST.search_line, "&", " ")
        search_line = string.replace(search_line, '"', " ")
        search_line = string.replace(search_line, ";", " ")
        if len(search_line) < 1:
            search_line = " "
        search_line = string.replace(search_line, " ", "zz20")
        sortmail = "search" + "x2jq" + REQUEST.search_where + "x2jq" \
                   + search_line + "x2jq" + REQUEST.search_when + "x2jq" \
                   + search_date + "x2jq" + flagged + "x2jqsortx2jq" + "DATE"

        try:
            if int(REQUEST.search_year)>1970 and 1<=int(REQUEST.search_day) and int(REQUEST.search_day)<=31:
                #search line OK
                return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' \
                                                 +  quote(REQUEST.IMAPName) + '&sortmail=' + sortmail)
            else:
                #error in date (must be > 1970)
                return REQUEST.response.redirect(self.absolute_url()+"/search?erreurDate=1" \
                                                 + '&IMAPName=' + quote(REQUEST.IMAPName))
        except:
            #error in search line (special chars)
            return REQUEST.response.redirect(self.absolute_url()+"/search?erreurLine=1" \
                                             + '&IMAPName=' + quote(REQUEST.IMAPName))


    security.declareProtected('View','searchAddressBook')  
    def searchAddressBook(self):
        """ search AdressBook product installed """
        try:
            abook = self.getAuthenticatedMemberCartable().objectValues('AddressBook2')[0]
        except:
            
            abook = None
                    
        if abook is not None:
            return abook
        else:
            try:
                abook = []
                if abook != []:
                    return abook[0][1]
                else:
                    return None
            except:
                return None

    security.declareProtected('View','setSignature')  
    def setSignature(self, signature):
        """ set personal mail signature """
        self.signature=signature

    security.declareProtected('View','getSignature')  
    def getSignature(self):
        """ return personal mail signature """
        return self.signature

    security.declareProtected('View','setAutoSaveSentMessage')  
    def setAutoSaveSentMessage(self, value):
        """ set AutoSaveSentMessage value for auto save sent-messages, boolean """
        self.autoSaveSentMessage = value

    security.declareProtected('View','getAutoSaveSentMessage')  
    def getAutoSaveSentMessage(self):
        """ return AutoSaveSentMessage value for auto save sent-messages, boolean """
        return self.autoSaveSentMessage

    security.declareProtected('View','setNbCharSubject')  
    def setNbCharSubject(self, value):
        """ set Nb Characters display for message Subject """
        self.nbCharSubject = value

    security.declareProtected('View','getNbCharSubject')  
    def getNbCharSubject(self):
        """ return Nb Characters display for message Subject """
        return self.nbCharSubject

    security.declareProtected('View','setListingSize')  
    def setListingSize(self, value):
        """ set the number of messages diplay on one page view """
        self.listingSize = value

    security.declareProtected('View','getListingSize')  
    def getListingSize(self):
        """ return the number of messages diplay on one page view """
        return self.listingSize

    security.declareProtected('View','setAutoViewAtt')  
    def setAutoViewAtt(self, value):
        """ set the autoViewAtt value, if 1 : auto-view html or pics under message """
        self.autoViewAtt = value

    security.declareProtected('View','getAutoViewAtt')  
    def getAutoViewAtt(self):
        """ return the autoViewAtt value """

        if hasattr(self, 'autoViewAtt'):
            return self.autoViewAtt
        else:
            return 0
        
    security.declareProtected('View','setTheme')  
    def setTheme(self, value):
        """ set the theme value """
        self.wmailtheme = value

    security.declareProtected('View','getTheme')  
    def getTheme(self):
        """ return the string them value """
        if hasattr(self, 'wmailtheme'):
            return self.wmailtheme
        else:
            return "theme1"

    security.declareProtected('View','getWMailStyle')  
    def getWMailStyle(self):
        """ return the theme value """
        theme = self.getTheme()
        if theme == "theme1":
            wMailStyle = self.wMailStyle1(self)
        if theme == "theme2":
            wMailStyle = self.wMailStyle2(self)
        if theme == "theme3":
            wMailStyle = self.wMailStyle3(self)
        if theme == "theme4":
            wMailStyle = self.wMailStyle4(self)
        if theme == "theme5":
            wMailStyle = self.wMailStyle5(self)
        if theme == "theme6":
            wMailStyle = self.wMailStyle6(self)
        return wMailStyle
    
    security.declareProtected('View','get_language')  
    def get_language(self):
        """return used language """
        if hasattr(self, 'wmailLang'):
            return self.wmailLang
        else:
            return "fr"

    security.declareProtected('View','setLang')  
    def setLang(self, value):
        """ set wmailLang value"""
        self.wmailLang = value

    security.declareProtected('View','set_language_and_config')  
    def set_language_and_config(self, REQUEST = None, RESPONSE = None):
        """ set language before start install """
        #set language
        self.setLang(REQUEST.wmailLanguage)
        
        #set default folders
        if REQUEST.wmailLanguage != "fr":
            self.get_lang(('common',), REQUEST)
        
        RESPONSE.redirect(self.absolute_url() + '/editConfiguration')

    security.declareProtected('View','gettext')  
    def gettext(self, text='', REQUEST=None):
        """ """
        if hasattr(self.aq_parent, 'gettext') and REQUEST is not None:
            return self.aq_parent.gettext(text, REQUEST)

        return text

    security.declareProtected('View','setOption')  
    def setOption(self, REQUEST):
        """ update WebMail options """
        try:
            if REQUEST.listingSize != self.getListingSize() and REQUEST.listingSize > 0:
                self.setListingSize(int(REQUEST.listingSize))
            if REQUEST.autoSaveSentMessage != self.getAutoSaveSentMessage():
                self.setAutoSaveSentMessage(REQUEST.autoSaveSentMessage)
            if REQUEST.nbCharSubject != self.getNbCharSubject() and REQUEST.nbCharSubject > 0:
                self.setNbCharSubject(int(REQUEST.nbCharSubject))
            if REQUEST.signature != self.getSignature():
                self.setSignature(REQUEST.signature)
            if REQUEST.autoViewAtt != self.getAutoViewAtt():
                self.setAutoViewAtt(REQUEST.autoViewAtt)
            if REQUEST.wmailtheme != self.getTheme():
                self.setTheme(REQUEST.wmailtheme)
            if REQUEST.wmailLang != self.get_language():
                self.setLang(REQUEST.wmailLang)
##            if REQUEST.mailFormat != self.getMailFormat():
##                self.mailFormat = REQUEST.mailFormat
            if self.getMailFormat() != 'text/plain':
                self.mailFormat = 'text/plain'


            return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' +  quote(REQUEST.IMAPName) + '&start=' + REQUEST.start)
        except:
            return REQUEST.response.redirect(self.absolute_url() + '/editOption?IMAPName=' +  quote(REQUEST.IMAPName) + '&start=' + REQUEST.start)

        
    security.declareProtected('View','setSortMail')  
    def setSortMail(self, value, REQUEST = None, RESPONSE = None):
        """ set the sorting for getting mail headers"""
        if value:
            RESPONSE.setCookie('sortmail', value , path = '/' + self.absolute_url(1) + '/')
        else:
            RESPONSE.expireCookie('sortmail', path = '/' + self.absolute_url(1) + '/')

    security.declareProtected('View','getSortMail')  
    def getSortMail(self, REQUEST = None):
        """ return the sorting for getting mail headers"""
        if REQUEST is not None:
            if REQUEST.has_key('sortmail'):
                return REQUEST['sortmail']
        return 'date'

    security.declareProtected('View','getFolderType')  
    def getFolderType(self, folderIMAPName):
        """ return the type of the folderIMAPName folder"""
        
    security.declareProtected('View','')  
    def setConfiguration(self, IMAPServer = "", IMAPPort = "", SMTPServer = "", SMTPPort = ""):
        """ set mail servers """
        self.IMAPHost = IMAPHost
        self.IMAPPort = IMAPPort
        self.SMTPHost = SMTPHost
        self.SMTPPort = SMTPPort

    security.declareProtected('View','_getFolder') 
    def _getFolder(self, IMAPName, con):
        if not hasattr(self, '_v_foldersCache'):
            self._v_foldersCache = SimpleCache(600)
        self._v_foldersCache.flush() 
        f = self._v_foldersCache.get(IMAPName)
        if f is not None:
            return f
        for f in self.getIMAPFolders(con):
            if f.getImapName() == IMAPName:
                self._v_foldersCache.set(f.getImapName(), f)
                break
        f = self._v_foldersCache.get(IMAPName)
        return f
        
    security.declareProtected('View','getFolder')      
    def getFolder(self, IMAPName, con):
        """ return foder instance """
        f = self._getFolder(IMAPName, con)
        if f:
            return f
        for f in self.getIMAPFolders(con):
            if IMAPName == f.getImapName():
                return f
            
    security.declareProtected('View','getInboxFolder')  
    def getInboxFolder(self):
        """ return the InboxFolder Id"""
        return IMAPFolder(self.getInboxIMAPName(), self.getInboxRdName(), self)

    security.declareProtected('View','getDraftFolder')  
    def getDraftFolder(self):
        """ return the Draft folder Id """
        return IMAPFolder(self.getDraftIMAPName(), self.getDraftRdName(), self)

    security.declareProtected('View','getSentMailFolder')  
    def getSentMailFolder(self):
        """ return the Sent Mail folder Id """

        return IMAPFolder(self.getSentMailIMAPName(), self.getSentMailRdName(), self)

    security.declareProtected('View','getTrashFolder')  
    def getTrashFolder(self):
        """ return the Trash Folder Id """

        return IMAPFolder(self.getTrashIMAPName(), self.getTrashRdName(), self)

    security.declareProtected('View','getInboxRdName')  
    def getInboxRdName(self):
        """ return redable name for inbox """

        return self.getDefaultFoldersNames()['INBOX'][1]

    security.declareProtected('View','getInboxIMAPName')  
    def getInboxIMAPName(self):
        """ return IMAP name for inbox folder"""

        return  self.getDefaultFoldersNames()['INBOX'][0]

    security.declareProtected('View','getSentMailRdName')  
    def getSentMailRdName(self):
        """ return redable name for sent mail folder """

        return self.getDefaultFoldersNames()['sent-mail'][1]

    security.declareProtected('View','getSentMailIMAPName')  
    def getSentMailIMAPName(self):
        """ return IMAP name for sent mail folder"""

        return self.getDefaultFoldersNames()['sent-mail'][0]

    security.declareProtected('View','getDraftRdName')  
    def getDraftRdName(self):
        """ return redable name for draft folder """

        return self.getDefaultFoldersNames()['draft'][1]

    security.declareProtected('View','getDraftIMAPName')  
    def getDraftIMAPName(self):
        """ return IMAP name for draft folder """

        return self.getDefaultFoldersNames()['draft'][0]

    security.declareProtected('View','getTrashRdName')  
    def getTrashRdName(self):
        """ return redable name for trash folder """

        return self.getDefaultFoldersNames()['trash'][1]

    security.declareProtected('View','getTrashIMAPName')  
    def getTrashIMAPName(self):
        """ return IMAP name for trash folder """

        return self.getDefaultFoldersNames()['trash'][0]

    security.declareProtected('View','setDefaultFoldersNames')  
    def setDefaultFoldersNames(self, value = {'INBOX':['INBOX', 'Bote de rception'],
                                      'sent-mail':['INBOX.sent-mail', 'Elments envoys'],
                                      'draft':['INBOX.brouillons', 'Brouillons'],
                                      'trash':['INBOX.Corbeille', 'Corbeille']}):
        """ set default folders """
        self.defaultFoldersNames = value

    security.declareProtected('View','getIMAPFolders')  
    def getIMAPFolders(self, con = None):
        """ return the liste of IMAP folders """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())

        imap_mailboxes = con.listFolders()

        prev_name = ""

        res = []
        for item in imap_mailboxes:
            readable_name = string.replace(item, "INBOX.", "")
            if string.find(item, "&") != (-1) :
                readable_name = string.replace(readable_name, "&AOk-", "")
                readable_name = string.replace(readable_name, "&AOA-", "")
                readable_name = string.replace(readable_name, "&AOI-", "")
                readable_name = string.replace(readable_name, "&AOg-", "")
                readable_name = string.replace(readable_name, "&-", "&")
                readable_name = string.replace(readable_name, "&AOc-", "")
                readable_name = string.replace(readable_name, "&APk-", "")
                readable_name = string.replace(readable_name, "&AOo-", "")
                readable_name = string.replace(readable_name, "&AOs-", "")
                readable_name = string.replace(readable_name, "&AO4-", "")
                readable_name = string.replace(readable_name, "&APQ-", "")
                readable_name = string.replace(readable_name, "&APM-", "")
                readable_name = string.replace(readable_name, "&APE-", "")
                readable_name = string.replace(readable_name, "&AOE-", "")
                readable_name = string.replace(readable_name, "&AMk-", "")
                

            if readable_name == "INBOX":
                readable_name = self.getInboxRdName()
            if readable_name == "inbox":
                readable_name = self.getInboxRdName()
                if self.getInboxIMAPName() != 'inbox':
                    self.defaultFoldersNames['INBOX'][0] = 'inbox'
            if readable_name == self.getSentMailIMAPName():
                readable_name = self.getSentMailRdName()

            name = string.split(readable_name, '.')[0]
            if name == prev_name and not (name == '' or prev_name == ''):
                readable_name = "  &nbsp; &nbsp; " + string.replace(readable_name, name, "")
            prev_name = name

            if not (name == '' or prev_name == ''):
                res.append(IMAPFolder(item, readable_name, self))


        # create default IMAP folder if not exist
        if not self.getSentMailIMAPName() in imap_mailboxes:
            con.connection.create(self.getSentMailIMAPName())
            res.append(IMAPFolder(self.getSentMailIMAPName(), self.getSentMailIMAPName(), self))
        if not self.getDraftIMAPName() in imap_mailboxes:
            con.connection.create(self.getDraftIMAPName())
            res.append(IMAPFolder(self.getDraftIMAPName(), self.getDraftIMAPName(), self))
        if not self.getTrashIMAPName() in imap_mailboxes:
            con.connection.create(self.getTrashIMAPName())
            res.append(IMAPFolder(self.getTrashIMAPName(), self.getTrashIMAPName(), self))
        # con.logout()

        return res

    security.declareProtected('View','get_attachement')  
    def get_attachement(self, IMAPId, AttachId = 1, IMAPName = "INBOX", REQUEST = None, RESPONSE = None, retour = 1, con = None):
        """ return the attachement for download"""
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())

        f = self.getFolder(IMAPName, con)   ## the folder
        m = f.getIMAPMessage(IMAPId, con)   ## the message

        #if attachment not rfc822 (mail message) type
        if string.find(AttachId, ".") == (-1):
            for a in m.getAttachements():
                if a.getId() == int(AttachId):
                    ret = a.download(REQUEST,RESPONSE)
                    con.logout()
                    return ret
            if retour:
                ret = self.view_message(IMAPName, IMAPId, REQUEST, con = con)
                con.logout()
                return ret
        
        con.logout()
        #if attachment is rfc822 (mail message) type and contains attachments
        li=string.split(AttachId, ".")

        # attachment we want is in first level rfc822 maill attachment
        # li is [attachement level, attachment number from original level, attachemnt number]
        if li[0] == "1":
            for a in m.getAttachements():
                if a.getId() == (int(li[1]) + 1):
                    virtual_message = a.render_mail_attachment()
                    for a2 in virtual_message.getAttachements():
                        if a2.getId() == int(li[2]):
                            return a2.download(REQUEST, RESPONSE)

                        
        # attchment is in second level
        # li is [attachement level, attachment number from original level, , attachment number from first level, attachemnt number]
        if li[0] == "2":
            for a in m.getAttachements():
                if a.getId() == (int(li[1]) + 1):
                    virtual_message = a.render_mail_attachment()
                    for a2 in virtual_message.getAttachements():
                        if a2.getId() == (int(li[2])+1):
                             virtual_message2 = a2.render_mail_attachment()
                             for a3 in virtual_message2.getAttachements(): 
                                 if a3.getId() == int(li[3]):
                                     return a3.download(REQUEST,RESPONSE)


    security.declareProtected('View','getQuota')  
    def getQuota(self, con = None):
        """ return % used quota on mailbox """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        try:
            res = con.getQuota()
        except:
            res = -1
        #con.logout()

        return res
        
    security.declareProtected('View','deleteMessages') 
    def deleteMessages(self, REQUEST=None, con = None):
        """ delete imap message on mail server"""

        if not REQUEST.has_key('IMAPIds'):
            return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' +  quote(REQUEST.IMAPName) + '&start=' + REQUEST.start)

        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())

        IMAPIds = REQUEST.IMAPIds
        
        if type(IMAPIds) is type(''):
                IMAPIds = [IMAPIds]

        if not REQUEST.has_key('start'):
            REQUEST.set('start', '0')

        if REQUEST.IMAPName:
            IMAPName = REQUEST.IMAPName
        else:
            IMAPName = "INBOX"

        #copy = 1 if moving message to trash
        _copy = 1
        quota = self.getQuota(con)
        
        if IMAPName == self.getTrashFolder().getImapName() or quota > 999:
            _copy = 0

        #copy=0 if delete message from trash or quota exceed for moving to rtash        
        f = self.getFolder(IMAPName, con)
        f.moveIMAPMessages(folderdest = "deleting", imapids = IMAPIds, copy = _copy, con = con)

        if self.getProtocol() == 'POP3':
            con.logout(force = 1)
        else:
            con.logout()

        return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' +  quote(REQUEST.IMAPName) + '&start=' + REQUEST.start)
        
    security.declareProtected('View','moveMessages')  
    def moveMessages(self, REQUEST = None, con = None):
        """ move messages from IMAP folder to another """

        if not REQUEST.has_key('IMAPIds'):
            return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' +  quote(REQUEST.IMAPName) + '&start=' + REQUEST.start)

        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())

        IMAPIds = REQUEST.IMAPIds
        
        if type(IMAPIds) is type(''):
                IMAPIds = [IMAPIds]

        if REQUEST.IMAPName:
            IMAPName = REQUEST.IMAPName
        else:
            IMAPName = "INBOX"
            
        f = self.getFolder(IMAPName, con)
        f.moveIMAPMessages(folderdest = REQUEST.move_to_folder, imapids = IMAPIds, con = con)
        con.logout()

        return REQUEST.response.redirect(self.absolute_url() + '/index_html?IMAPName=' +  quote(REQUEST.IMAPName) + '&start=' + REQUEST.start)

    security.declareProtected('View','')  
    def getPreviousAndNextMessagesIds(self, folderName, sortmail, IMAPId, con = None):
        """ return next IMAP id for sorted mails """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
        res = con.getPreviousAndNextMessagesIds(folderName = folderName, sortmail = sortmail, IMAPId = IMAPId)
        #con.logout()
        return res

    security.declareProtected('View','saveDraft')  
    def saveDraft(self, REQUEST, con = None):
        """ save message in draft Folder """
        if con is None:
            con = self.getPasserelle()
            con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
            con.login(self.getIMAPLogin(), self.getIMAPPassword())
            
        listeAtt = []
        if (REQUEST.has_key('IMAPName') and REQUEST.has_key('IMAPId')):
            ##check if forwarded message or draft message with attachments
            try:
                f = self.getFolder(REQUEST['IMAPName'], con)   ## the folder
                m = f.getIMAPMessage(REQUEST['IMAPId'], con)   ## the message
                if REQUEST.has_key('forwarded_attachment'):
                    for a in m.getAttachements():
                        if str(a.getId()) in REQUEST['forwarded_attachment']:
                            listeAtt.append(a)      ## we append him to the list of attachements
            except:
                pass
            
        _to = [REQUEST.to]
        _cc = [REQUEST.cc]
        _bcc = [REQUEST.bcc]
        _headers={'subject':REQUEST.subject}

        ##### We add attachment
        for i in range(1, 100):
            attKey = 'attFile%d' % i
            if REQUEST.has_key(attKey):
                if (type(REQUEST[attKey]) is not type('')) and (REQUEST[attKey].filename):
                    att = REQUEST[attKey]
                    filename = att.filename
                    try:
                        filename = string.split(filename,'\\')[-1]
                    except:
                        pass
                    data = att.read()
                    content_type = OFS.content_types.guess_content_type(
                        name = filename, body = data,
                        default = 'application/octet-stream')[0]
                    listeAtt.append(Attachement(id = str(i), filename = filename, content_type = content_type, size = len(data), data = data))

                    
        message = IMAPMessage(expediteur = (self.getNomFrom(), self.getMailFrom()), subject = REQUEST.subject,
                              date = "lundi", body = REQUEST.body,
                              destinataires = {'to':_to, 'cc':_cc, 'bcc':_bcc},
                              attachements = listeAtt,
                              headers = _headers)

        message.headers['date'] =  self.ZopeTime().rfc822()
        
        message_brut = message.raw_message()
        message_brut = string.replace(message_brut, "\n", "\r\n")

        draft_folder = self.getDraftFolder().getImapName()
        
        draft_id = con.writeMessage(draft_folder, message_brut)
        try:
            con.setFlag(folderName = draft_folder, IMAPId = draft_id, flag = "draft") 
        except:
            pass
    
        con.logout()

        return REQUEST.response.redirect(self.absolute_url() + '/index_html')

    security.declareProtected('View','sendMail')  
    def sendMail(self, to = '', cc = '', bcc = '', subject = '[pas de sujet]', body = '' ,redirection_url = 'edit_message', REQUEST = None, con = None, mailFormat=None):
        """ send a mail """
        _to = [to]
        _cc = [cc]
        _bcc = [bcc]
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        ImapName = "INBOX"
        if REQUEST.has_key('IMAPName'):
            ImapName = REQUEST['IMAPName']
            
        st = '0'
        
        if REQUEST.has_key('start'):
            st = REQUEST['start']
        
        if to + cc + bcc == '':
            if redirection_url:
                return REQUEST.RESPONSE.redirect(REQUEST['URL1'] + '/' + redirection_url + \
                                                 '?IMAPName=' + quote(ImapName) + '&start=' + st + \
                                                 '&portal_status_message=Erreur%20message%20sans%20destinataire.')
            else:
                return
            
        ##### We add attachment
        listeAtt = []
        for i in range(1, 100):
            attKey = 'attFile%d' % i
            if REQUEST.has_key(attKey):
                if type(REQUEST[attKey]) is not type('') and REQUEST[attKey].filename:
                   att = REQUEST[attKey]
                   if hasattr(att,'filename'):
		    filename = att.filename
                    try:
                        filename = string.split(filename,'\\')[-1]
                    except:
                        pass
                    
                    data = att.read()
                    content_type = OFS.content_types.guess_content_type(
                        name = filename, body = data,
                        default = 'application/octet-stream')[0]
                    listeAtt.append(Attachement(id = str(i), filename = filename, content_type = content_type, size = len(data), data = data))
            
        ########## Attachment from cartableFolder or cartableGroupe ###########

        if REQUEST.has_key('attUrls') and REQUEST['attUrls'] != '':
            try:
                for url in string.split(REQUEST['attUrls'], ';'):
                    obj = self.unrestrictedTraverse(url)
                    filename = obj.title_or_id()
                    if obj.meta_type == 'Document':
                        data = obj.cooked_text
                        content_type = 'text/html'
                    else:
                        data = obj.getRawData()
                        content_type = obj.content_type
                    listeAtt.append(Attachement(id = str(i), filename = filename, content_type = content_type, size = len(data), data = data))
            except:
                pass
            
        ### add the attchement from the original message when forward or send from draft
        if (REQUEST.has_key('IMAPName') and REQUEST.has_key('IMAPId') and REQUEST.has_key('forward')) \
               or (REQUEST.has_key('IMAPName') and REQUEST['IMAPName']==self.getDraftFolder().getImapName() \
               and REQUEST.has_key('IMAPId')) :

            ## attForward is the list of attachement id to be forwarded
            f = self.getFolder(REQUEST['IMAPName'], con)   ## the folder
            m = f.getIMAPMessage(REQUEST['IMAPId'], con)   ## the message
            if REQUEST.has_key('forwarded_attachment'):
                for a in m.getAttachements():
                    if str(a.getId()) in REQUEST['forwarded_attachment']:
                        listeAtt.append(a)


	_headers={'subject':subject, 'date':self.ZopeTime().rfc822()}
        message = IMAPMessage(expediteur=(self.getNomFrom(), self.getMailFrom()),
                              subject=subject,
                              date="lundi",
                              body=body,
                              destinataires={'to':_to, 'cc':_cc, 'bcc':_bcc},
                              replyTo=self.getReplyTo(),
                              attachements=listeAtt,     
                              headers=_headers,
                              content_type=self.getMailFormat(mailFormat))

        raw_message = message.raw_message()
        if REQUEST.has_key('autoSaveSentMessage') and REQUEST.autoSaveSentMessage == "oui":       
            #copy to sent-mail on server
            message_brut = string.replace(raw_message, "\n", "\r\n")
            if con is None:
                con = self.getPasserelle()
                con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
                con.login(self.getIMAPLogin(), self.getIMAPPassword())
            con.writeMessage(folderName = self.getSentMailFolder().getImapName(), raw_message = message_brut)
            #con.logout()
        erreur = 1
        try:
            smtp_connection = smtplib.SMTP(self.getSMTPServer())
            _recipients = []
            for item in ['to', 'cc', 'bcc']:
                for address in message.destinataires[item]:
                    spl = ","
                    if string.find(address, ",") != -1:
                        spl = ","
                    else:
                        spl = ";"
                    for people in string.split(address, spl):
                        peop = string.strip(people)
			if string.find(peop, "@") == -1 and (len(peop) > 2):
				peop = peop + "@" + self.getDomain()
                        if len(peop) > 2:
                            _recipients.append(peop)

            mailfrom = string.replace(self.getNomFrom(), '.', '-') + ' <' + self.getMailFrom() + '>'
            smtp_connection.sendmail(mailfrom, _recipients, raw_message)
            smtp_connection.quit()
        except:
            erreur = 2
        
        if erreur == 2:
            if redirection_url:
                REQUEST.set('IMAPName', quote(ImapName))
                REQUEST.set('start', st)
                REQUEST.set('error_message',"Une erreur est survenue, votre message n'a pas pu etre envoye, veulliez retentez d'envoyer ce message")
                destinat = {'to':string.join(message.destinataires['to'], ','), 'cc':string.join(message.destinataires['cc'], ','), \
                            'bcc':string.join(message.destinataires['bcc'], ',')}
                return self.edit_message(subject = subject, body = body, destinataires = destinat,attachements = listeAtt, REQUEST = REQUEST)
        else:
            if REQUEST.has_key('IMAPName') and REQUEST.has_key('IMAPId') and REQUEST.has_key('reply'):
                if con is None:
                    con = self.getPasserelle()
                    con.connect(self, server = self.getIMAPServer(), port = self.getIMAPPort())
                    con.login(self.getIMAPLogin(), self.getIMAPPassword())
                con.setFlag(folderName = REQUEST['IMAPName'], IMAPId = REQUEST['IMAPId'], flag = "anwser")
                #con.logout()
            if redirection_url:    
                return REQUEST.RESPONSE.redirect(self.absolute_url() + '/index_html?IMAPName=' + quote(ImapName) + '&start=' + st)

    security.declareProtected('View','processId')  
    def processId(self, filename):
        if hasattr(self.aq_parent, 'processId'):
            return self.aq_parent.processId(filename)
        id=''
        # Generate valid  Zope id
        id = filename
        id = id[max(
            string.rfind(id, '/'), 
            string.rfind(id, '\\'), 
            string.rfind(id, ':')
            )+1:]
        id = string.replace(id, ' ', '_')
        id = string.replace(id, '(', '_')
        id = string.replace(id, ')', '_')
        id = string.replace(id, '%', '_')
        id = string.replace(id, '!', '_')
        id = string.replace(id, '&', '_')
        id = string.replace(id, '?', '_')
        id = string.replace(id, '', 'e')
        id = string.replace(id, '', 'e')
        id = string.replace(id, '', 'e')
        id = string.replace(id, '', 'a')
        id = string.replace(id, '', 'a')
        id = string.replace(id, '', 'u')
        id = string.replace(id, '', 'i')
        id = string.replace(id, '', 'i')
        id = string.replace(id, '', 'u')
        id = string.replace(id, '', 'u')
        id = string.replace(id, '', 'o')
        id = string.replace(id, "'", '_')
        if id:
            return id
        else:
            return None

    security.declareProtected('View','setNomAttachementWithInt')  
    def setNomAttachementWithInt(self, nomFichier, ind):
        """
           Rajoute un indice a noFichier pour ne pas avoir
           d'erreur lors de setObject (attachement.txt ==> attachement_2.txt)
        """
        data = string.split(nomFichier, '.')
        nomCourt = data[0]
        if nomCourt[-2:] == '_' + str(ind):
            nomCourt = nomCourt[:-1] + str(ind + 1) 
        else:
            nomCourt = nomCourt + '_' + str(ind)
        data[0] = nomCourt
        res = string.join(data, '.')
        return res

    security.declareProtected('View','move_attachement_to_cartable')  
    def move_attachement_to_cartable(self, idsattach = [], REQUEST = None, RESPONSE = None):
        """Wrapper around manage_copyObjects, avoids management interface"""

        if not idsattach:
            try:
                return RESPONSE.redirect(self.absolute_url() + \
                                         "/view_message?portal_status_message=Erreur%20Aucun%20attachement%20slectionn&IMAPId=" + \
                                         REQUEST['IMAPId'] + "&sortmail=" + REQUEST.sortmail + "&start=" + REQUEST.start + \
                                         "&IMAPName=" + quote(REQUEST.IMAPName))
            except:
                return RESPONSE.redirect(self.absolute_url() + \
                                         "/view_message?portal_status_message=Erreur%20Aucun%20attachement%20slectionn&IMAPId=" + \
                                         REQUEST['IMAPId'] + "&sortmail=" + REQUEST.sortmail[0] + "&start=" + \
                                         REQUEST.start[0] + "&IMAPName=" + quote(REQUEST.IMAPName))

        ref = self.aq_parent
        if type(idsattach) is type(''):
            idsattach = [idsattach]
        for id in idsattach:
           
            try:
                imap_name , imap_id , attach_id , attach_filename , attach_content_type = string.split(id, '9:o')
            except:
                imap_name , imap_id , attach_id , attach_filename  = string.split(id, '9:o')
                attach_content_type = "type"

            ## IMAPId, AttachId=1, IMAPName 
            item = self.get_attachement(imap_id, attach_id, imap_name, REQUEST, RESPONSE, retour = 0)            
            
            nomFichier = self.processId(attach_filename)
            nomFichierTmp = nomFichier
            ind = 1
            while nomFichierTmp in ref.objectIds():
                ind = ind + 1
                nomFichierTmp = self.setNomAttachementWithInt(nomFichier, ind)
            nomFichier = nomFichierTmp
           
            if attach_content_type[:6] == 'image/':
                try:
                    from Products.PTKBase.PortalExtImage import manage_addPortalExtImage
                    manage_addPortalExtImage(ref, fn = nomFichier, title = '', descr = '', file = cStringIO.StringIO(item), \
                                             preview = '', content_type = attach_content_type, create_prev = 1, maxx = '256', \
                                             maxy = '256',ratio = 1, permission_check = 0)
                except:
                    ref.manage_addImage(nomFichier, file = cStringIO.StringIO(item), title = nomFichier, precondition = '', \
                                        content_type = attach_content_type)
                    
            else:
                try:
                    from Products.PTKBase.PortalExtFile import manage_addPortalExtFile
                    manage_addPortalExtFile(ref, fn = nomFichier, title = '', descr = '', file = cStringIO.StringIO(item), \
                                            content_type = attach_content_type)
                except:
                    ref.manage_addFile(nomFichier, file = cStringIO.StringIO(item), title = nomFichier, precondition = '', \
                                       content_type = attach_content_type)
        if type(REQUEST.start) is type([]):
            s = REQUEST.start[0]
        else:
            s = REQUEST.start
        return RESPONSE.redirect(self.absolute_url() + "/view_message?IMAPName=" + quote(imap_name) + \
                                 "&IMAPId=" + imap_id + "&start="+ s + "&portal_status_message=Dpos%20dans%20le%20cartable")
        

    security.declareProtected('View','compose_direct_mail_with_attachment')  
    def compose_direct_mail_with_attachment(self, conteneur, ids = None, destinataires = [], title = '', body = '', REQUEST = None, RESPONSE = None):
        """ compose mail with cartable item """
        erreur = 0
        if title:
            subject = title
        elif body:
            subject = body[0:30]
        else:
            subject = "document(s) de mon cartable"

        mailable = []
        unmailable = []

        listeObject = map( (lambda x, conteneur = conteneur : getattr(conteneur,x )),filter(lambda x, ids = ids : x in ids , conteneur.objectIds()))
        
        for obj in listeObject:
            if obj.isPrincipiaFolderish and not obj.meta_type in ['ZipFolder','Repertoire Cartable','Album photo']:
                listeObject = listeObject + obj.objectValues()
        nombreObjets = len(listeObject)

        for obj in listeObject:
            if obj.meta_type not in ['Image', 'Fichier', 'Link', 'Nouvelle', 'Document', 'ZipFolder', 'Repertoire Cartable','Album photo']:
                unmailable.append(obj.id)
                listeObject.remove(obj)
        if not listeObject:
            return [], unmailable

        # Formatage Expediteur
        sender = (self.getNomFrom(), self.getMailFrom())
        
        # Les Headers
        _headers = {'subject':subject, 'date':self.ZopeTime().rfc822()}

        # La Liste des attachements pour la creation de l'objet IMAPMessage
        listeAtt = []
        i = 0
        for obj in listeObject:
            filename = ''
            content_type = ''
            data = ''
            try:
                if hasattr(obj, 'title') and obj.title:
                    filename = obj.title
                else:
                    filename = obj.id
                if obj.meta_type == 'Link':
                    content_type = 'text/plain'
                    data = obj.title + ' : ' + obj.remote_url + '\n' + obj.text + '\n\n'
                elif obj.meta_type == 'Nouvelle':
                    content_type = 'text/plain'
                    data =  obj.title + ' : ' + obj.description + '\n' + obj.text + '\n\n'
                elif obj.meta_type == 'Document':
                    if obj.text_format in ["structured-text","plaintext"]:
                        content_type = 'text/plain'
                    else:
                        content_type = 'text/html'
                    data = obj.text
                    if string.find(filename,'.') == -1:
                        filename = filename + '.txt'
                elif obj.meta_type in ['Image', 'Fichier']:
                    content_type = obj.content_type
                    data = obj.getRawData()
                elif obj.meta_type in ['ZipFolder','Repertoire Cartable','Album photo']:
                    data = obj.aq_parent.exportObject(ids=[obj.id])
                    content_type = 'application/zip'

                if filename and content_type and data:
                    i = i + 1
                    listeAtt.append(Attachement(id = str(i), filename = filename, content_type = content_type, size = len(data), data = data)) 
                    mailable.append(obj.id)
                else:
                    unmailable.append(obj.id)
            except:
                import traceback
                traceback.print_exc()
                unmailable.append(obj.id)
                pass
                # "erreur du type de fichier / dossier"

        message = IMAPMessage(expediteur = sender,
                              subject = subject,
                              date = _headers['date'],
                              body = body + "\n\n--\n" + self.getSignature() ,
                              destinataires = {'to': destinataires,'cc':[],'bcc':[] },
                              attachements = listeAtt,     
                              headers = _headers)
        raw_message = message.raw_message()
        
        smtp_connection = smtplib.SMTP(self.getSMTPServer())
        try:
            smtp_connection.sendmail(sender[0] + ' <' + sender[1] + '>', message.destinataires['to'], raw_message)
        except:
            erreur = 2
        smtp_connection.quit()

        return mailable,unmailable


    security.declareProtected('View','add_to_addressbook')  
    def add_to_addressbook(self, name = ' ', email = ' ', IMAPName='', IMAPId = '', start = '', REQUEST = None):
        """ find and add to addressBook """

        if REQUEST.has_key('mailBookMail'):
            new_contact = string.replace(REQUEST.mailBookName, ",", " ") + "," + string.replace(REQUEST.mailBookMail, ", ", " ")
            self.adPeopleMailBook(new_contact = new_contact)
        else:
            addressbook = self.searchAddressBook()
            addressbook.addABContactExterieur(nom = name, email = email)
        
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/view_message?IMAPName=' + quote(IMAPName) + \
                                      '&IMAPId=' + IMAPId + '&start=' + start + '&contactAdded=1')


    security.declareProtected('View','PutBodyInCartable')  
    def PutBodyInCartable(self, body = '', subject = '', REQUEST = None, RESPONSE = None):
        """Put a body message in your cartable"""
        titlep = self.processId(subject)
        ref = self.aq_parent
        id = str(DateTime.DateTime().millis())
        try:
            from Products.PTKBase.PortalExtFile import manage_addPortalExtFile
            manage_addPortalExtFile(ref, fn = id, title = titlep, descr = '', file = cStringIO.StringIO(body), content_type = 'text/html')
        except:
            ref.manage_addFile(id, file = cStringIO.StringIO(body), title = titlep, precondition = '', content_type = 'text/html')
                
        return REQUEST.RESPONSE.redirect(self.absolute_url() + '/index_html?portal_status_message=Depose%20dans%20le%20cartable%20reussie')

    security.declareProtected('View','page_navigation_menu')  
    def page_navigation_menu(self, start = "0", folder_id = None, nb_messages_in_mailbox = 1):
            """ return the navigation page for batching messages list"""

            menu = ""                        #### Le string contenant le menu resultat
            start = int(start)
            folder_id = quote(folder_id)
            
            # Nombre de mail par page
            listingSize = int(self.listingSize)
            
            # Nombre totale de mail dans la boite
            try:
                nbMessages = int(nb_messages_in_mailbox)
            except:
                return menu
            if not nbMessages:
                return menu
            
            # Taille d'un tranche de page
            tailleFixeNbPage = 10
            
            # Nombre de pages total
            nbPages, reste = divmod(nbMessages, listingSize)
            if reste:
                nbPages += 1
            
            # Nombre de Tranche total
            nTrancheMax, reste2 = divmod(nbPages, tailleFixeNbPage)
            if reste2:
                nTrancheMax += 1
            
            # Numero de page en fonction du start
            e, f = divmod(start, listingSize) 
            noPage = e + 1
            
            # Numero de la tranche
            np, reste3 = divmod(noPage, tailleFixeNbPage)
            if reste3:
                np += 1
            
            # Calcul de la borne infieur et superieur de la tranche np    
            if np != nTrancheMax:
                sup = tailleFixeNbPage * np
                inf = sup - (tailleFixeNbPage - 1)
            else:
                sup = nbPages
                inf = tailleFixeNbPage * np - (tailleFixeNbPage - 1)
            
            # les << de l'affichage
            if start >= 1:
                prec = start - listingSize
                menu += " <a href='%s/index_html?IMAPName=%s&start=%d' class=\"navigation\">%s</a> " % (self.absolute_url(), folder_id, prec, " &laquo;&laquo; ")
            
            # Construction de la tranche
            for indice in range(inf, sup + 1):
                newstart = (indice - 1) * listingSize
        
                if indice == noPage:
                    # La page ou on est
                    menu += " <a href='%s/index_html?IMAPName=%s&start=%d' class=\"hotnavigation\">%d</a> " % (self.absolute_url(), folder_id, newstart, indice)
                else:
                    # les autres pages
                    menu += " <a href='%s/index_html?IMAPName=%s&start=%d' class=\"navigation\">%d</a> " % (self.absolute_url(), folder_id, newstart, indice)
            
            # les >> de l'affichage
            sup2, mod = divmod(nbMessages, listingSize)
            if not mod:
                sup2 = sup2 - 1
            
            if start >= (sup2*listingSize) :
                return menu
            else:
                suiv = start + listingSize
                menu += " <a href='%s/index_html?IMAPName=%s&start=%d' class=\"navigation\">%s</a> " % (self.absolute_url(), folder_id, suiv, " &raquo;&raquo; ")
            return menu   
            

    def __bobo_traverse__(self, REQUEST, name):
        """ redifined method for get attachment name when download """

        try:
            if hasattr(self, name) and not REQUEST.has_key('AttachId'):
                return getattr(self, name)
        except AttributeError:
            pass
        try:
            if REQUEST.has_key('AttachId'):
                return self.get_attachement
        except:
            pass
        REQUEST.RESPONSE.notFoundError("%s\n" % (name))

